<?php

namespace MoSfSyncSalesforce\API;

use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Services\Utils;

/**
 * This library is miniOrange Authentication Service.
 *
 * Contains Request Calls to Customer service.
 */

class CustomerAPI {
	public $email;
	public $phone;

	/*
	 * * Initial values are hardcoded to support the miniOrange framework to generate OTP for email.
	 * * We need the default value for creating the first time,
	 * * As we don't have the Default keys available before registering the user to our server.
	 * * This default values are only required for sending an One Time Passcode at the user provided email address.
	 */
    private $defaultCustomerKey = "16555";
    private $defaultApiKey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";

	function mo_sf_sync_create_customer() {
		$url = pluginConstants::HOSTNAME . '/moas/rest/customer/add';

		$current_user = wp_get_current_user();
		$this->email = get_option ( 'mo_sf_sync_admin_email' );
		$password = get_option ( 'mo_sf_sync_admin_password' );

		$fields = array (
				'areaOfInterest' => 'Object Data Sync For Salesforce Plugin',
				'email' => $this->email,
				'password' => $password
		);
		$field_string = json_encode ( $fields );

		$headers = array("Content-Type"=>"application/json","charset"=>"UTF-8","Authorization"=>"Basic");

		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '10',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		
		return $response;
	}

	function mo_sf_sync_get_customer_key() {
		$url = pluginConstants::HOSTNAME . "/moas/rest/customer/key";

		$email = get_option ( "mo_sf_sync_admin_email" );

		$password = get_option ( "mo_sf_sync_admin_password" );

		$fields = array (
				'email' => $email,
				'password' => $password
		);
		$field_string = json_encode ( $fields );

		
		$headers = array("Content-Type"=>"application/json","charset"=>"UTF-8","Authorization"=>"Basic");
		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '15',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		
		return $response;
	}

	function mo_sf_sync_check_customer() {
		$url = pluginConstants::HOSTNAME . "/moas/rest/customer/check-if-exists";

		$email = get_option ( "mo_sf_sync_admin_email" );

		$fields = array (
				'email' => $email
		);
		$field_string = json_encode ( $fields );

		$headers = array("Content-Type"=>"application/json","charset"=>"UTF-8","Authorization"=>"Basic");
		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '10',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		return $response;
	}

	function mo_sf_sync_check_customer_ln(){

		$url = pluginConstants::HOSTNAME. '/moas/rest/customer/license';

		$customerKey = get_option ( 'mo_sf_sync_admin_customer_key' );

		$apiKey = get_option ( 'mo_sf_sync_admin_api_key' );
		$currentTimeInMillis = round(microtime(true) * 1000);
		$stringToHash = $customerKey . number_format($currentTimeInMillis, 0, '', '') . $apiKey;
		$hashValue = hash("sha512", $stringToHash);
		$currentTimeInMillis = number_format ( $currentTimeInMillis, 0, '', '' );
		$fields = '';
		$fields = array(
			'customerId' => $customerKey,
			'applicationName' => 'wp_object_sync_for_salesforce_plan'
		);
		$field_string = json_encode($fields);

		$headers = array(
			"Content-Type" => "application/json",
			"Customer-Key" => $customerKey,
			"Timestamp" => $currentTimeInMillis,
			"Authorization" => $hashValue
		);

		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '10',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		return $response;
	}

	function mo_sf_sync_vl($code,$active) {
		$url = "";
		if($active)
			$url = pluginConstants::HOSTNAME . '/moas/api/backupcode/check';
		else
			$url = pluginConstants::HOSTNAME . '/moas/api/backupcode/verify';

		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_sf_sync_admin_customer_key' );

		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_sf_sync_admin_api_key' );

		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );

		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );

		$currentTimeInMillis = number_format ( $currentTimeInMillis, 0, '', '' );
		$fields = '';

		// *check for otp over sms/email

		$fields = array (
				'code' => $code ,
				'customerKey' => $customerKey,
				'additionalFields' => array(
					'field1' => home_url()
				)

		);

		$field_string = json_encode ( $fields );

		$headers = array(
			"Content-Type" => "application/json",
			"Customer-Key" => $customerKey,
			"Timestamp" => $currentTimeInMillis,
			"Authorization" => $hashValue
		);

		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '10',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		return $response;
	}

	function mo_sf_sync_forgot_password($email) {
		$url = pluginConstants::HOSTNAME . '/moas/rest/customer/password-reset';

		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_sf_sync_admin_customer_key' );

		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_sf_sync_admin_api_key' );

		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );

		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );

		$fields = '';

		// *check for otp over sms/email
		$fields = array (
				'email' => $email
		);

		$field_string = json_encode ( $fields );
		$headers = array(
			"Content-Type" => "application/json",
			"Customer-Key" => $customerKey,
			"Timestamp" => $currentTimeInMillis,
			"Authorization" => $hashValue
		);
		$args = array(
			'method' => 'POST',
			'body' => $field_string,
			'timeout' => '10',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers
		);
		$response = Utils::mo_sf_sync_wp_remote_call($url, $args);
		return $response;

	}
	function mo_sf_sync_get_timestamp() {
		$url = pluginConstants::HOSTNAME . '/moas/rest/mobile/get-timestamp';
		$response = Utils::mo_sf_sync_wp_remote_call($url, array());
		return $response;
	}
}
