<?php

namespace MoSfSyncSalesforce\API;

use MoSfSyncSalesforce\Handler\AuthorizationHandler;
use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\IAPI\ISalesforce;

class Salesforce implements ISalesforce{
	use Instance;

	private $AuthorizationUrl;
	private $loginUrl;
	private $tokenendpoint;
	private $AuthorizationHandler;
	private $refresh_token;
	public $api_version='v53.0';
	private $sf_object = 'User';
	public $instance_url;
	public $access_token;

	public function __construct() {
		$this->AuthorizationHandler = AuthorizationHandler::instance();
		$token                      = $this->AuthorizationHandler->mo_sf_sync_get_access_token();
		if ( $token ) {
			$this->access_token  = $token[PluginConstants::ACCESS_TOKEN];
			$this->refresh_token = $token[PluginConstants::REFRESH_TOKEN];
			$this->instance_url  = $token['instance_url'];
		}
	}

	private function mo_sf_sync_get_api_endpoint( $object_type ) {

		return $this->instance_url . '/services/data/' . $this->api_version . '/sobjects/' . $object_type;
	}

	/**
	 * @return mixed
	 */
	public function mo_sf_sync_get_access_token() {
		return $this->access_token;
	}

	/**
	 * @param  mixed  $access_token
	 */
	public function mo_sf_sync_set_access_token( $access_token ): void {
		$this->access_token = $access_token;
	}

	public function mo_sf_sync_get_objects(){
		$access_token = $this->access_token;

		$url = $this->instance_url.'/services/data/' . $this->api_version . '/sobjects/';
		$headers = array (
			"Authorization" => "Bearer ".$access_token
		);
		$response = $this->mo_sf_sync_get_request($url, $headers);
		$result = $this->mo_sf_sync_check_token($url,$headers,$response);
		if(!is_wp_error($result)){
			return $result['sobjects'] ?? array();
		}
		return $result;
	}

	public function mo_sf_sync_get_fields($object){

		$access_token = $this->access_token;
		$url = $this->instance_url.'/services/data/' . $this->api_version . "/sobjects/".$object."/describe/";
		$headers = array (
			"Authorization" => "Bearer ".$access_token
		);
		$response = $this->mo_sf_sync_get_request($url, $headers);
		return $this->mo_sf_sync_check_token($url,$headers,$response);
	}

	public function mo_sf_sync_get_api_versions()
	{
		$access_token = $this->access_token;
		$url = $this->instance_url.'/services/data';
		$headers = array (
			"Authorization" => "Bearer ".$access_token
		);
		$response = $this->mo_sf_sync_get_request($url, $headers);
		return $this->mo_sf_sync_check_token($url,$headers,$response);
	}
	
	public function mo_sf_sync_check_token($url,$headers,$response,$method='GET',$body=""){
		if(is_array($response) && count($response) && isset($response[0]['errorCode']) && $response[0]['errorCode']=='INVALID_SESSION_ID'){
			$data = $this->AuthorizationHandler->mo_sf_sync_get_new_token();
			if($data == "Please install miniOrange on your connected salesforce environment"|| $data == "Salesforce Instance URL Not Found, please Authorize Again" || $data = "Cannot connect to salesforce, please Authorize Again!" || !isset($data["access_token"]))
				return $data;
			
			$this->access_token=$data["access_token"];
			$headers["Authorization"] = "Bearer ".$this->access_token;
			if($method!='GET'){		
				return $this->mo_sf_sync_post_request($url,$method,$headers,$body);
			}
			$response= $this->mo_sf_sync_get_request($url, $headers);
			if(is_array($response ) && count($response) && isset($response[0]['errorCode']) && $response[0]['errorCode']=='INVALID_SESSION_ID')
				return "Invalid session";
			else return $response;
		}
		else return $response;
	}

	public function mo_sf_sync_update_record($id,$object,$body){
		$access_token = $this->access_token;
		$url = $this->instance_url.'/services/data/' . $this->api_version . "/sobjects/".$object."/".$id;
		$headers = array(
			"Authorization" => "Bearer ".$access_token,
			"Content-Type" => "application/json"
		);
		$response = $this->mo_sf_sync_post_request($url,"PATCH",$headers,wp_json_encode( $body ));
		return $this->mo_sf_sync_check_token($url,$headers,$response,"PATCH",wp_json_encode( $body ));
	}

	public function mo_sf_sync_create_record($object,$body){
		$access_token = $this->access_token;
		$url = $this->instance_url.'/services/data/' . $this->api_version . "/sobjects/".$object;
		$headers = array(
			"Authorization" => "Bearer ".$access_token,
			"Content-Type" => "application/json"
		);
		$response = $this->mo_sf_sync_post_request($url,"POST",$headers,wp_json_encode( $body ));
		return $this->mo_sf_sync_check_token($url,$headers,$response,"POST",wp_json_encode( $body ));
	}

	public function mo_sf_sync_get_request($url,$headers){		
		$response = wp_remote_get($url, array(
			'headers' => $headers,
			'timeout'     => 10
		));
		if ( is_array( $response ) && ! is_wp_error( $response ) ) {			
			return json_decode($response["body"],true);
		} else {
			return $response->get_error_messages();
		}
	}

	public function mo_sf_sync_post_request($url,$method,$headers,$body){
		$response = wp_remote_post($url,array(
			"method" => $method,
			"body" => $body,
			"headers" => $headers,
			'timeout'     => 15
		));
		if ( is_array( $response ) && ! is_wp_error( $response ) ) {			
			return json_decode($response["body"],true);
		} else {
			return $response->get_error_messages();
		}
	}
	public function  mo_sf_sync_send_composite_request($composite_url,$headers,$composite_request_body)
	{
		$composite_response = $this->mo_sf_sync_post_request($composite_url, 'POST', $headers, $composite_request_body);
		return $this->mo_sf_sync_check_token($composite_url, $headers, $composite_response,  "POST", $composite_request_body );
	}
}