<?php

namespace MoSfSyncSalesforce\Handler;

use MoSfSyncSalesforce\API\CustomerAPI;
use MoSfSyncSalesforce\Customer;
use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Services\Utils;
class AccountSetupHandler {
    use Instance;

    public static function instance()
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self;
		}
        Utils::mo_sf_sync_show_message();
        Utils::mo_sf_sync_show_feedback_message();
		return self::$_instance;
	}

    public function mo_sf_sync_save_settings(){
        $option = sanitize_text_field($_POST['option']);
        
        switch ($option){
            case 'mo_sf_sync_account_setup_option':{
                if(check_admin_referer('mo_sf_sync_account_setup_option'))
                    $this->mo_sf_sync_account_setup();
                break;
            }
            case 'mo_sf_sync_go_back_from_license_page_option':{
                if(check_admin_referer('mo_sf_sync_go_back_from_license_page_option'))
                    $this->mo_sf_sync_go_back_from_license_page();
                break;
            }
            case 'mo_sf_sync_remove_account_option':{
                if(check_admin_referer('mo_sf_sync_remove_account_option'))
                    $this->mo_sf_sync_remove_account();
                break;
            }
            case 'mo_sf_sync_account_registration':{
                if(check_admin_referer('mo_sf_sync_account_registration'))
                    $this->mo_sf_sync_account_registration_option();
                break;
            }

        }
    }

    private function mo_sf_sync_is_extension_installed($extension_name) {
        if  (in_array  ($extension_name, get_loaded_extensions())) {
            return 1;
        } else
            return 0;
    }

    private function mo_sf_sync_account_setup(){

        if (!$this->mo_sf_sync_is_extension_installed('curl')) {
            Utils::mo_sf_sync_show_error_message("ERROR: PHP cURL extension is not installed or disabled. Login failed.");
            return;
        }

        $email = '';
        $password = '';
        if (empty($_POST['account_email']) || empty($_POST['account_pwd'])) {
            Utils::mo_sf_sync_show_error_message("All the fields are required. Please enter valid entries.");
            return;
        } else if (Customer::mo_sf_sync_checkPasswordPattern(strip_tags($_POST['account_pwd']))) {
            Utils::mo_sf_sync_show_error_message("Minimum 6 characters should be present. Maximum 15 characters should be present. Only following symbols (!@#.$%^&*-_) should be present.");
            return;
        } else {
            $email = sanitize_email($_POST['account_email']);
            $password = stripslashes(strip_tags($_POST['account_pwd']));
        }

        update_option('mo_sf_sync_admin_email', $email);
        update_option('mo_sf_sync_admin_password', $password);
        $customer = new CustomerAPI();
        $content = $customer->mo_sf_sync_get_customer_key();
        if (!$content)
            return;

        $customerKey = json_decode($content, true);

        if (json_last_error() == JSON_ERROR_NONE) {
            update_option('mo_sf_sync_admin_customer_key', $customerKey['id']);
            update_option('mo_sf_sync_admin_api_key', $customerKey['apiKey']);
            update_option('mo_sf_sync_customer_token', $customerKey['token']);
            if (!empty($customerKey['phone'])) {
                update_option('mo_sf_sync_admin_phone', $customerKey['phone']);
            }
            update_option('mo_sf_sync_admin_password', '');
            update_option('mo_sf_sync_registration_status', PluginConstants::EXISTING_USER);
            delete_option('mo_sf_sync_verify_customer');
            Utils::mo_sf_sync_show_success_message('Successfully Logged In');

            } else {
                Utils::mo_sf_sync_show_error_message("Invalid username or password. Please try again.");
            }

            update_option('mo_sf_sync_admin_password', '');
            
    }
  
    private function mo_sf_sync_go_back_from_license_page(){
        Customer::mo_sf_sync_deactivate();
        add_option('mo_sf_sync_registration_status', PluginConstants::REMOVED_ACCOUNT);
    }

    private function mo_sf_sync_remove_account(){
        Customer::mo_sf_sync_deactivate();
        add_option('mo_sf_sync_registration_status', PluginConstants::REMOVED_ACCOUNT);
    }

    private function mo_sf_sync_account_registration_option(){

        if (!$this->mo_sf_sync_is_extension_installed('curl')) {
            Utils::mo_sf_sync_show_error_message("ERROR: PHP cURL extension is not installed or disabled. Login failed.");
            return;
        }
        $email = '';
        $password = '';
        $confirm_password = '';

        if ((array_key_exists('account_email_reg',$_POST) && empty($_POST['account_email_reg'])) ||(array_key_exists('reg_account_pwd',$_POST) && empty($_POST['reg_account_pwd']) )|| (array_key_exists('confirm_account_pwd',$_POST)  && empty($_POST['confirm_account_pwd']))) {
            Utils::mo_sf_sync_show_error_message("All the fields are required. Please enter valid entries.");
            return;
        } else if (Customer::mo_sf_sync_checkPasswordPattern(strip_tags($_POST['reg_account_pwd']))) {
            Utils::mo_sf_sync_show_error_message("Minimum 6 characters should be present. Maximum 15 characters should be present. Only following symbols (!@#.$%^&*-_) should be present.");
            return;
        }else {
            $email = sanitize_email($_POST['account_email_reg']);
            $password = stripslashes(strip_tags($_POST['reg_account_pwd']));
            $confirm_password = stripslashes(strip_tags($_POST['confirm_account_pwd']));
        }
        update_option('mo_sf_sync_admin_email', $email);

        if (strcmp($password, $confirm_password) == 0) {
            update_option('mo_sf_sync_admin_password', $password);
            $customer = new CustomerAPI();
                $content = json_decode($customer->mo_sf_sync_check_customer(), true);
                
                if (!is_null($content)) {
                    if (strcasecmp($content['status'], 'CUSTOMER_NOT_FOUND') == 0) {

                        $response = create_customer();
                        
                        if (is_array($response) && array_key_exists('status', $response) && $response['status'] == 'success') {
                            Utils::mo_sf_sync_show_success_message('Successfully Logged In');
                            wp_safe_redirect(admin_url("/admin.php?page=mo_sf_sync&tab=account_setup"));
                            exit;
                        }
                    } else {
                        $response = get_current_customer();
                        if (is_array($response) && array_key_exists('status', $response) && $response['status'] == 'success') {
                            Utils::mo_sf_sync_show_success_message('Successfully Logged In');
                            wp_safe_redirect(admin_url("/admin.php?page=mo_sf_sync&tab=account_setup"));
                            exit;
                        }
                        
                    }
                }

            }    
            else {
                Utils::mo_sf_sync_show_error_message("Passwords don't match!");
                delete_option('mo_sf_sync_verify_customer');
            }
            return;
        }

}
function create_customer() {
    $customer    = new CustomerAPI();
    $customerKey = json_decode( $customer->mo_sf_sync_create_customer(), true );
    if(!is_null($customerKey)){
        $response = array();
        if ( strcasecmp( $customerKey['status'], 'CUSTOMER_USERNAME_ALREADY_EXISTS' ) == 0 ) {
            $api_response = get_current_customer();
            if($api_response){
                $response['status'] = "success";
            }
            else
                $response['status'] = "error";

        } else if ( strcasecmp( $customerKey['status'], 'SUCCESS' ) == 0 ) {
            update_option( 'mo_sf_sync_admin_customer_key', $customerKey['id'] );
            update_option( 'mo_sf_sync_admin_api_key', $customerKey['apiKey'] );
            update_option( 'mo_sf_sync_customer_token', $customerKey['token'] );
            update_option( 'mo_sf_sync_free_version', 1 );
            update_option( 'mo_sf_sync_admin_password', '' );
            update_option( 'mo_sf_sync_registration_status', '' );
            delete_option( 'mo_sf_sync_verify_customer' );
            $response['status']="success";
            return $response;
        }

        update_option( 'mo_sf_sync_admin_password', '' );
        return $response;
    }
}

function get_current_customer() {
    $customer    = new CustomerAPI();
    $content     = $customer->mo_sf_sync_get_customer_key();
   
    if(!is_null($content)){
        $customerKey = json_decode( $content, true );

        $response = array();
        if ( json_last_error() == JSON_ERROR_NONE ) {
            update_option( 'mo_sf_sync_admin_customer_key', $customerKey['id'] );
            update_option( 'mo_sf_sync_admin_api_key', $customerKey['apiKey'] );
            update_option( 'mo_sf_sync_customer_token', $customerKey['token'] );
            update_option( 'mo_sf_sync_admin_password', '' );

            delete_option( 'mo_sf_sync_verify_customer' );
            $response['status'] = "success";
            return $response;
        } else {

            Utils::mo_sf_sync_show_success_message('You already have an account with miniOrange. Please enter a valid password.');
            $response['status'] = "error";
            return $response;
        }
    }
}