<?php

namespace MoSfSyncSalesforce\Handler;

use MoSfSyncSalesforce\API\Salesforce;
use MoSfSyncSalesforce\Customer;
use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\MoSfSync;
use MoSfSyncSalesforce\Services\Utils;

class AjaxHandler{
	use Instance;
	private $Salesforce;

	public static function instance()
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self;
			self::$_instance->Salesforce = new Salesforce();
			self::$_instance->ObjectSyncSfToWp = new ObjectSyncSfToWp();
		}
		return self::$_instance;
	}

	public function mo_sf_sync_set_settings() {

		if ( ! current_user_can( 'administrator' ) ) {
			wp_send_json_error( "Not an administrator" );
			return;
		}

		if( isset( $_POST['push'] )) {
			$dataProcessingHandler = DataProcessingHandler::instance();
			$res = $dataProcessingHandler->mo_sf_sync_push_to_salesforce(sanitize_text_field( $_POST['push'] ));
			if(is_wp_error($res)){
				wp_send_json_error(implode(', ',$res->get_error_messages()));
			} else
				wp_send_json_success( $res );
			return;
		}

		if(isset($_POST['integration_trial_request'])){
			set_transient('mo_sf_sync_integration_trial_notice_dismiss_time',true,604800);
			wp_send_json_success(true);
		}

		if(isset($_POST['trial_request'])){
			set_transient('mo_sf_sync_normal_trial_notice_dismiss_time',true,604800);
			wp_send_json_success(true);
		}

		if( isset( $_POST['object'] )) {
			$res= $this->Salesforce->mo_sf_sync_get_fields(sanitize_text_field( $_POST['object'] ));
			if($res == "Please install miniOrange on your connected salesforce environment" || $res == "Cannot connect to salesforce, please Authorize Again!" || $res == "Salesforce Instance URL Not Found, please Authorize Again")
				wp_send_json_error(array('error_description' => $res));
			if(is_wp_error($res)){
				wp_send_json_error(implode(', ',$res->get_error_messages()));
			} else
				wp_send_json_success( $res );
			return;
		}

		if(isset($_POST['wp_object'])){
			$wp_object = sanitize_text_field($_POST['wp_object']);
			
			$wp_object_fields = $this->ObjectSyncSfToWp->mo_sf_sync_get_wp_obj_fields($wp_object);
			$wp_object_fields_keys = array();

			foreach ($wp_object_fields as $key=>$value)
				$wp_object_fields_keys[]= $value['key'];
			
			if(is_wp_error($wp_object_fields))
				wp_send_json(implode(',',$wp_object_fields->get_error_messages()));
			else
				wp_send_json_success($wp_object_fields_keys);
		}

		if(isset($_POST['query'])){
			$users = new \WP_User_Query( array(
				'search'         => '*'.sanitize_text_field($_POST['query']).'*',
				'search_columns' => array(
					'user_login',
					'user_nicename',
					'user_email',
					'user_url',
				),
				'number' => 10
			) );
			$users=$users->get_results();
			wp_send_json_success($users);
			return;
		}

		if ( ! isset( $_POST['option'] ) || ! isset( $_POST['nonce_'] ) || ! isset( $_POST['tab'] ) ) {
			wp_send_json_error( "Invalid Request" );
		}

		check_ajax_referer( sanitize_text_field($_POST['option']),'nonce_' );
		$tab = sanitize_text_field( $_POST['tab'] );


		$this->mo_sf_sync_handle_ajax_request();
	}

	public function mo_sf_sync_handle_ajax_request() {
		$option = sanitize_text_field( $_POST['option'] );
		$this->mo_sf_sync_route_to_correct_handler( $option );
	}

	private function mo_sf_sync_route_to_correct_handler( $option ) {
		switch ( $option ) {
			case 'mo_sf_sync_client_config_option':
			{
				$map = Utils::mo_sf_sync_sanitize_and_index( $_POST['data'] );
				if(isset($map[PluginConstants::CLIENT_SECRET]))
					$map[PluginConstants::CLIENT_SECRET] = AuthorizationHandler::mo_sf_sync_encrypt_data($map[PluginConstants::CLIENT_SECRET], hash("sha256",$map[PluginConstants::CLIENT_ID]));
				Utils::mo_sf_sync_save_settings( PluginConstants::CONFIG_OBJECT, $map );
				wp_send_json_success( "Saved" );
				break;
			}
			case 'mo_sf_sync_app_provisioning_config_option':
			{
				$this->mo_sf_sync_handle_save_configuration('provision');
				break;

			}
			case 'mo_sf_sync_contact_us_query_option':
			{
				$this->mo_sf_sync_get_support();
				break;
			}
		}
	}

	public function mo_sf_sync_handle_config_object_save($postdata){
		if(!isset($postdata['client_id']) || empty($postdata['client_id'])) {
			$data['env_link'] = esc_url_raw($postdata['env_link']);
			$data['env_select'] = sanitize_text_field($postdata['env_select']);
			$data['app_type'] = 'preconnected';
			if(empty($data['env_link']))
				Utils::mo_sf_sync_show_error_message('Save Unsuccessful because no Environment selected');
			set_transient(PluginConstants::TRANSIENT_CONFIG_OBJECT,$data);
			Utils::mo_sf_sync_save_settings(PluginConstants::CONFIG_OBJECT,$data);
			Utils::mo_sf_sync_show_success_message('Selected Environment Saved Successfully !');
		} else {
			$data['env_link'] = esc_url_raw($postdata['env_link']);
			$data['env_select'] = sanitize_text_field($postdata['env_select']);
			$data['client_id'] = sanitize_text_field($postdata['client_id']);
			$data['client_secret'] = sanitize_text_field($postdata['client_secret']);
			$data['client_secret'] = AuthorizationHandler::mo_sf_sync_encrypt_data($data['client_secret'],hash("sha256", $data['client_id']));
			$data['redirect_uri'] = esc_url_raw($postdata['redirect_uri']);
			$data['app_type'] = 'manual';
			set_transient(PluginConstants::TRANSIENT_CONFIG_OBJECT,$data);
			Utils::mo_sf_sync_save_settings(PluginConstants::CONFIG_OBJECT,$data);
			delete_option(PluginConstants::CONNECTION_TYPE);
			Utils::mo_sf_sync_show_success_message('Configuration Saved Successfully !');
		}
	}

	private function mo_sf_sync_handle_save_configuration($suffix){
		$map = Utils::mo_sf_sync_sanitize_and_index( $_POST['data'] );
		Utils::mo_sf_sync_save_settings( 'mo_sf_sync_'.$suffix,$map );
		if(isset($map[PluginConstants::AUTO_USER_UPDATE]) && $map[PluginConstants::AUTO_USER_UPDATE] === 'on'){
			$handler = MoSfSync::instance();
			$handler->mo_sf_sync_load_real_time_sync_hooks();
		}
		wp_send_json_success( "Saved" );
	}

	public function mo_sf_sync_get_support(){
		$map = Utils::mo_sf_sync_sanitize_and_index( $_POST['data'] );
		$email    = $map['mo_sf_sync_contact_us_email'];
		$query    = $map['mo_sf_sync_contact_us_query'];
		if(empty($email) || empty($query)){		
			wp_send_json_error("Empty email or query");
		} else {
			$support = new Customer();
			$response = $support->mo_sf_sync_submit_contact_us($email, "", $query, false);
			$response=="Query submitted."?wp_send_json_success($response):wp_send_json_error($response);
		}
	}
}