<?php
namespace MoSfSyncSalesforce\Handler;

use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\Services\Utils;
use MoSfSyncSalesforce\API\Salesforce;
use MoSfSyncSalesforce\Helper\mo_sf_sync_pre_connected_app_Enum;
use MoSfSyncSalesforce\Services\AuditDB;
class AuthorizationHandler {

	use Instance;
	private $configurations;
	private $access_token;
	private $refresh_token;
	private $api_version = '';
	private $sf_object = 'Users';
	private $instance_url;
	private $auditHandler;

	public static function instance()
	{
		
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self;
			self::$_instance->mo_sf_sync_set_configurations(self::$_instance->mo_sf_sync_client_info());
			self::$_instance->mo_sf_sync_set_access_token(Utils::mo_sf_sync_get_settings(PluginConstants::SF_RESPONSE_OBJECT));
			self::$_instance->auditHandler = AuditDB::instance();
		}
		return self::$_instance;
	}


	private function mo_sf_sync_client_info(){
		return maybe_unserialize( Utils::mo_sf_sync_get_settings(PluginConstants::CONFIG_OBJECT));
	}

	/**
	 * @return mixed
	 */
	public function mo_sf_sync_handle_code(){
		if(isset($_REQUEST['code'])){
			$client = $this->mo_sf_sync_client_info();
			if(array_key_exists('preconnected',$_REQUEST) && $_REQUEST['preconnected'] == true){
				update_option(PluginConstants::CONNECTION_TYPE,'automatic');
				$manual_app_config = get_option(PluginConstants::CONFIG_OBJECT);
				if(isset($manual_app_config)){
					unset($manual_app_config[PluginConstants::CLIENT_ID]);
					unset($manual_app_config[PluginConstants::CLIENT_SECRET]);
					unset($manual_app_config[PluginConstants::REDIRECT_URI]);
					update_option(PluginConstants::CONFIG_OBJECT,$manual_app_config);
				}
					
				$client[PluginConstants::CLIENT_ID] = mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_ID ;
				$client[PluginConstants::CLIENT_SECRET]= mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_SECRET;
				$client[PluginConstants::CLIENT_SECRET] = AuthorizationHandler::mo_sf_sync_encrypt_data($client[PluginConstants::CLIENT_SECRET], hash("sha256",$client[PluginConstants::CLIENT_ID]));
				$client[PluginConstants::REDIRECT_URI] = mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_REDIRECT_URI ;
			}
			else
                update_option(PluginConstants::CONNECTION_TYPE,'manual');
						
			$sf_env_url = !empty($client[PluginConstants::ENVIRONMENT_LINK])?$client[PluginConstants::ENVIRONMENT_LINK]:'https://test.salesforce.com';
		
			if(!empty($client[PluginConstants::CLIENT_SECRET]))
				$client[PluginConstants::CLIENT_SECRET] = AuthorizationHandler::mo_sf_sync_decrypt_data($client[PluginConstants::CLIENT_SECRET],hash("sha256",$client[PluginConstants::CLIENT_ID]));
			if(empty($client[PluginConstants::CLIENT_ID]) || empty($client[PluginConstants::CLIENT_SECRET]) || empty($client[PluginConstants::REDIRECT_URI])){
				wp_die("<b>[MOSFSYNCERR001]:</b> Invalid client credentials. Save valid credentials.",'miniOrange Authorization Error');
				return;
			}
		
			$code = sanitize_text_field(urldecode($_REQUEST['code']));
			if(!empty($code)){
				$env = 'login';
				if(array_key_exists('preconnected',$_REQUEST) && $_REQUEST['preconnected'] == true)
					$body = array("client_id"=>mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_ID,"client_secret"=>mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_SECRET,"redirect_uri"=>mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_REDIRECT_URI,"grant_type"=>"authorization_code","code"=>$code);
				else
					$body = array("client_id"=>$client[PluginConstants::CLIENT_ID],"client_secret"=>$client[PluginConstants::CLIENT_SECRET],"redirect_uri"=>$client[PluginConstants::REDIRECT_URI],"grant_type"=>"authorization_code","code"=>$code);
				
				$body = http_build_query($body);
				$authURl =$sf_env_url."/services/oauth2/token";
				$this->auditHandler->mo_sf_sync_add_log("Authentication Call", "", "", "", 'Info', 'Call URL: '.$authURl, $wp_object = 'user');
				$res = wp_remote_post($authURl,['headers'=>array('content-type'=>'application/x-www-form-urlencoded'), 'body'=>$body]);
				
				if(is_wp_error($res)){
					wp_die('<b>[MOSFSYNCERR003]:</b> '.esc_html($res->get_error_message()).'</br></br>Note: This issue can occur due to active caching on your site. Please try again after clearing the cache. If the issue still persists, please contact us at <b>salesforcesupport@xecurify.com</b>','miniOrange Authorization Error');
					exit();
					return;
				}
				$token = json_decode($res['body'],true);
				if((array_key_exists(PluginConstants::REFRESH_TOKEN,$token))&&(array_key_exists(PluginConstants::ACCESS_TOKEN,$token))&&(array_key_exists('instance_url',$token))){
					$sf_response = ['access_token'=>sanitize_text_field($token[PluginConstants::ACCESS_TOKEN]),
									'refresh_token'=>sanitize_text_field($token[PluginConstants::REFRESH_TOKEN]),
						            'instance_url'=>esc_url($token['instance_url'])];
					update_option(PluginConstants::SF_RESPONSE_OBJECT, $sf_response);
					$this->mo_sf_sync_set_access_token(Utils::mo_sf_sync_get_settings(PluginConstants::SF_RESPONSE_OBJECT));
					$salesforce = new Salesforce();
					$lead_fields = $salesforce->mo_sf_sync_get_fields('lead',true);
					$get_objects = $salesforce->mo_sf_sync_get_objects(true);
					$processed_objects = [];
					foreach ($get_objects as $key => $value){
						$processed_objects[$key] = ['name'=>$value['name'],'label'=>$value['label']];
					}
					Utils::mo_sf_sync_set_transient('transient_lead_object',$lead_fields,100);
				    Utils::mo_sf_sync_set_transient('transient_get_object',$processed_objects,0);
					
				}
				else if(array_key_exists("error_description",$token)){			
					wp_die('<b>[MOSFSYNCERR002]:</b> '.wp_kses_post($token['error_description']),'miniOrange Authorization Error');
				} else {
					wp_die("<b>[MOSFSYNCERR004]:</b> Unknown error occurred",'miniOrange Authorization Error');
				}
		
				echo '<div style="color: #3c763d;
				background-color: #dff0d8; padding:2%;margin-bottom:20px;text-align:center; border:1px solid #AEDB9A; font-size:18pt; border-radius:10px;margin-top:17px;">Your connection with Salesforce is successful</div>
				<div style="display:block;text-align:center;margin-bottom:4%;margin-top:-3.5rem;"><div class="animation-ctn">
				<div class="icon icon--order-success svg">
					<svg xmlns="http://www.w3.org/2000/svg" width="154px" height="154px">  
					  <g fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"> 
						<circle cx="77" cy="77" r="72" style="stroke-dasharray:480px, 480px; stroke-dashoffset: 960px;" stroke-width="7"></circle>
						<circle id="colored" fill="#fff" cx="77" cy="77" r="72" style="stroke-dasharray:480px, 480px; stroke-dashoffset: 960px;"></circle>
						<polyline class="st0" stroke="rgba(76, 175, 80, 1)" stroke-width="7" points="43.5,77.8 63.7,97.9 112.2,49.4 " style="stroke-dasharray:100px, 100px; stroke-dashoffset: 200px;"/>   
					  </g> 
					</svg>
				  </div>
		  </div><style>
			.animation-ctn{
				text-align:center;
				margin-top:5em;
			}
			@-webkit-keyframes checkmark {
				0% {
					stroke-dashoffset: 100px
				}
				100% {
					stroke-dashoffset: 200px
				}
			}
			@-ms-keyframes checkmark {
				0% {
					stroke-dashoffset: 100px
				}
				100% {
					stroke-dashoffset: 200px
				}
			}
			@keyframes checkmark {
				0% {
					stroke-dashoffset: 100px
				}
				100% {
					stroke-dashoffset: 0px
				}
			}
			@-webkit-keyframes checkmark-circle {
				0% {
					stroke-dashoffset: 480px
				}
				100% {
					stroke-dashoffset: 960px;
				}
			}
			@-ms-keyframes checkmark-circle {
				0% {
					stroke-dashoffset: 240px
				}
				100% {
					stroke-dashoffset: 480px
				}
			}
			@keyframes checkmark-circle {
				0% {
					stroke-dashoffset: 480px 
				}
				100% {
					stroke-dashoffset: 960px
				}
			}
			@keyframes colored-circle { 
				0% {
					opacity:0
				}
				100% {
					opacity:100
				}
			}
			.inlinesvg .svg svg {
				display: inline
			}
			.icon--order-success svg polyline {
				-webkit-animation: checkmark 0.25s ease-in-out 0.7s backwards;
				animation: checkmark 0.25s ease-in-out 0.7s backwards
			}
			.icon--order-success svg circle {
				-webkit-animation: checkmark-circle 0.6s ease-in-out backwards;
				animation: checkmark-circle 0.6s ease-in-out backwards;
			}
			.icon--order-success svg circle#colored {
				-webkit-animation: colored-circle 0.6s ease-in-out 0.7s backwards;
				animation: colored-circle 0.6s ease-in-out 0.7s backwards;
			} 
			</style></div>';
		echo '
		 <br>
		 <div style="background: #d5e2ff;padding: 0.2rem 1.5rem;border-radius: 5px;
				font-size: 1.1em;line-height: 1.4;border-left: 5px solid #77a1ff;">
				<b>Note:</b> Click on <b>Next</b> to map Salesforce and WordPress fields as next step to configure the sync between salesforce and WordPress
		</div>
		 ';

		 echo '
		 <div style="margin:3%;display:block;text-align:center;">
		 <input style="margin-right: 10px;background: linear-gradient(0deg,rgb(14 42 71) 0,rgb(26 69 138) 100%)!important;cursor: pointer;font-size:15px;border: none;font-size: 1.1em;padding:0.7rem 1.5rem;border-radius:5px;color: #FFF;cursor: pointer;text-decoration: none;"
		 type="button" value="Next" onclick="close_and_redirect_to_field_mapping();" >
		 </div>
		 <script>
		window.opener.test_configuration_status()
		function close_and_redirect_to_field_mapping(){
			window.opener.redirect_to_field_mapping();
			self.close();
		} 

		function close_and_redirect_to_advance_sync(){
			window.opener.close_and_redirect_to_advance_sync();
			self.close();
		}
		</script> 
		 ';
				exit();
			}
			if(isset($_REQUEST['error'])){
				$token['error_description'] = wp_kses_post($_REQUEST['error_description']);
				wp_die('<b>[MOSFSYNCERR005]:</b> '.esc_html($token['error_description']),'miniOrange Authorization Error');
			}
		}
	}

	public function mo_sf_sync_connect_to_salesforce(){
		if(array_key_exists('app_type',$_GET) && isset($_GET['app_type'])){
			$client = $this->mo_sf_sync_client_info();
			$sf_env_url = !empty($client[PluginConstants::ENVIRONMENT_LINK])?$client[PluginConstants::ENVIRONMENT_LINK]:'https://test.salesforce.com';
			if($_GET['app_type'] == 'preconnected'){
				$client_id = mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_ID;
				$redirect_uri = mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_REDIRECT_URI;
				$url = $sf_env_url.'/services/oauth2/authorize?response_type=code&state='.home_url().'&grant_type=authorization_code&scope=api refresh_token&client_id='.$client_id.'&redirect_uri='.$redirect_uri;
			}
			else if($_GET['app_type'] == 'manual'){
				$client_id = $client[PluginConstants::CLIENT_ID];
				$redirect_uri = $client[PluginConstants::REDIRECT_URI];
				$url = $sf_env_url.'/services/oauth2/authorize?response_type=code&grant_type=authorization_code&scope=api refresh_token&client_id='.$client_id.'&redirect_uri='.$redirect_uri;
			}
			wp_redirect($url);
			exit;
		}
	}

    public function mo_sf_sync_get_new_token() {
		$client = $this->mo_sf_sync_get_configurations();
		$connection_type= get_option(PluginConstants::CONNECTION_TYPE);

		if($connection_type == 'automatic'){
			$client[PluginConstants::CLIENT_ID] =  mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_ID;
			$client[PluginConstants::CLIENT_SECRET] = mo_sf_sync_pre_connected_app_Enum::PRE_CONNECTED_APP_CLIENT_SECRET;
			$sf_env_url = $client[PluginConstants::ENVIRONMENT_LINK];
		}
		else{
			$client[PluginConstants::CLIENT_SECRET] = AuthorizationHandler::mo_sf_sync_decrypt_data($client[PluginConstants::CLIENT_SECRET],hash("sha256",$client[PluginConstants::CLIENT_ID]));
			$sf_env_url = $client[PluginConstants::ENVIRONMENT_LINK];
		}
		
		$tokens = $this->mo_sf_sync_get_access_token();
		if(empty($tokens) || !is_array($tokens) || !isset($tokens["refresh_token"]))
			return 'Cannot connect to salesforce, please Authorize Again!';
		
		$refresh_token = sanitize_text_field($tokens["refresh_token"]);
		
		if(isset($sf_env_url))
        	$url = $sf_env_url.'/services/oauth2/token';
		else
			return 'Salesforce Instance URL Not Found, please Authorize Again';
        $data = array (
            "grant_type" => PluginConstants::REFRESH_TOKEN,
            'client_id' => $client[PluginConstants::CLIENT_ID],
            'client_secret' => $client[PluginConstants::CLIENT_SECRET],
            'refresh_token' => $refresh_token
        );
        $response = wp_remote_post( $url, array(
            'body' => $data
        ));
		if ( is_wp_error( $response ) ) {
			$error_message = $response->get_error_message();
			return "Something went wrong: $error_message";
		} else {
			$body= json_decode($response["body"],true);
			if(isset($body["access_token"])){
				$this->access_token=sanitize_text_field($body["access_token"]);
				$option = get_option(PluginConstants::SF_RESPONSE_OBJECT);
				$new_option = array_merge((array) $option, (array) $body);
				update_option(PluginConstants::SF_RESPONSE_OBJECT, $new_option);	
				return $this->access_token;
			} 
			else return $body;
		}
    }
	/*
	 * returns @client_id , @client_secret @tenant_id @redirect_uri @spo_id
	 * */
	public function mo_sf_sync_get_configurations() {
		return $this->configurations;
	}

	/**
	 * @param mixed $configurations
	 */
	public function mo_sf_sync_set_configurations( $configurations ): void {
		$this->configurations = $configurations;
	}

	/**
	 * @return mixed
	 */
	public function mo_sf_sync_get_access_token() {
		return $this->access_token;
	}

	/**
	 * @param  mixed  $access_token
	 */
	public function mo_sf_sync_set_access_token( $access_token ): void {
		$this->access_token = $access_token;
	}

	/**
	 * @return mixed
	 */
	public function mo_sf_sync_get_refresh_token() {
		return $this->refresh_token;
	}

	/**
	 * @param  mixed  $refresh_token
	 */
	public function mo_sf_sync_set_refresh_token( $refresh_token ): void {
		$this->refresh_token = $refresh_token;
	}

	/**
     * @param string $data - the key=value pairs separated with &
     * @return string
     */
    public static function mo_sf_sync_encrypt_data($data, $key) {
        $key    = openssl_digest($key, 'sha256');
        $method = 'aes-128-ecb';
        $strCrypt = openssl_encrypt ($data, $method, $key,OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING);
        return base64_encode($strCrypt);
    }


    /**
     * @param string $data - crypt response from Sagepay
     * @return string
     */
    public static function mo_sf_sync_decrypt_data($data, $key) {
        $strIn = base64_decode($data);
        $key    = openssl_digest($key, 'sha256');
        $method = 'AES-128-ECB';
        $ivSize = openssl_cipher_iv_length($method);
        $iv     = substr($strIn,0,$ivSize);
        $data   = substr($strIn,$ivSize);
        $clear  = openssl_decrypt ($data, $method, $key, OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING, $iv);

        return $clear;
    }

}