<?php

namespace MoSfSyncSalesforce\Handler;

use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Services\DBUtils;
use MoSfSyncSalesforce\Services\Utils;
use MoSfSyncSalesforce\API\Salesforce;
use MoSfSyncSalesforce\Services\AuditDB;

if( defined('ultimatemember_version') ){
	include_once WP_PLUGIN_DIR . '/ULTIMATE-MEMBER/includes/class-init.php';
}

class DataProcessingHandler {
    use Instance;
    private $ajaxHandler;
    private $dbUtils;
    public static function instance()
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self;
            self::$_instance->Salesforce = new Salesforce();
            self::$_instance->enable_audit_logs = get_option(PluginConstants::AUDIT_LOGS);
            self::$_instance->ajaxHandler = AjaxHandler::instance();
            self::$_instance->dbUtils = DBUtils::instance();
            self::$_instance->audit = AuditDB::instance();
            self::$_instance->ObjectSyncSfToWp = new ObjectSyncSfToWp();
		}
		return self::$_instance;
	}

    public function mo_sf_sync_process_raw_mapping_data($post_data){
		$field_mapping = array(
			'field_map' => array(
				'field_type' => array(),
				'field_mapping' => array(),
				'name_constraint' => array(),
				'type_constraint' => array(),
				'size_constraint' => array()
			)
		);

		if(empty($post_data)){
			Utils::mo_sf_sync_show_error_message('Empty Data cannot be saved. ');
			return;
		}
		if(isset($post_data['updatable_status']) && in_array('non_updatable',$post_data['updatable_status'])){
			Utils::mo_sf_sync_show_error_for_updatable_fields('Record Cannot Be Saved Becasue You Tried To Map Some Non-Updatable Fields.');
			return;
		}
		$salesforce_object = sanitize_text_field($post_data['object_select']);
		$wordpress_object = sanitize_text_field($post_data['wp_object_select']);
		$sync_direction = isset($post_data['sync_wp_to_sf']) ? 'sync_wp_to_sf':'sync_sf_to_wp';

		$post_data['wordpress_fields'] = array_values($post_data['wordpress_fields']);

		$field_mapping['field_map']['field_type'] = $post_data['field_types'];
		foreach($post_data['sf_fields'] as $key=>$value){
			$field_mapping['field_map']['field_mapping'][$value] = $post_data['wordpress_fields'][$key]; 
		}
		$field_mapping['field_map']['name_constraint'] = $post_data['name_constraint'];
		$field_mapping['field_map']['type_constraint'] = $post_data['type_constraint'];
		$field_mapping['field_map']['size_constraint'] = $post_data['maxlength_constraint'];
		$this->dbUtils->mo_sf_sync_save_mapping_in_object_table($salesforce_object,$wordpress_object,$sync_direction,$field_mapping);
		Utils::mo_sf_sync_show_success_message('Data Saved Successfully !');
	}

    public function mo_sf_sync_get_mapping($id, $wp_object='user'){				
		
		$db = DBUtils::instance();
		$map = $db->mo_sf_sync_get_all_mapping_data();

		if(empty($map))
			return NULL;

		$direction = "wp_to_sf";
		if(isset($map['sync_sf_to_wp']) && $map['sync_sf_to_wp'] == 1)
			$direction = "sf_to_wp";

		$object = isset($map['salesforce_object']) ? $map['salesforce_object'] : '';

		if(empty($object) || empty($map['field_mapping'])) 
			return NULL;

		$object_table_structure = $this->ObjectSyncSfToWp->mo_sf_sync_get_wp_db_table_structure($wp_object);
		$id_field = $object_table_structure['id_field'];
		$wprecord_method = $object_table_structure['content_methods']['read'];
		
		if(str_ends_with($wprecord_method,"_by"))
			$wprecord = (array) $wprecord_method($id_field, $id)->data;
		else if($wprecord_method === 'get_posts')
			$wprecord = get_post($id, ARRAY_A); 

		$wpmeta_method = $object_table_structure['meta_methods']['read'];
		$wpmeta = (array)$wpmeta_method($id);
		
		$field_map = maybe_unserialize($map['field_mapping'])['field_map'];

		$fieldmap_iterator = 0;

		foreach($field_map['field_mapping'] as $key => $value) {
			
			$sf_field_type = $field_map['type_constraint'][$fieldmap_iterator];
			$mapping_type = $field_map['field_type'][$fieldmap_iterator];

			if($sf_field_type == 'picklist') {
				if(is_array($value)) { 
					$new_map[$key] = $this->mo_sf_sync_decide_value_based_on_conditions($value, $wprecord, $wpmeta);
					if(empty($new_map[$key]))
						unset($new_map[$key]);
				} else {
					$new_map[$key] = $value;
				}
			} else {
				if($mapping_type == 'static') {
					$new_map[$key] = $value;
				} else {
					$new_map[$key] = $this->mo_sf_sync_read_value_from_wp_data($value, $wprecord, $wpmeta);
				}
			}
			$fieldmap_iterator++;
		}

		return array("object" => $object, "body" => $new_map, "wp_object" => $wp_object, "direction" => $direction);
	}

	public function mo_sf_sync_read_value_from_wp_data($wp_attr, $wprecord, $wpmeta) {
		if((!empty($wprecord[$wp_attr]) || (isset($wpmeta[$wp_attr]) && $wpmeta[$wp_attr][0] )))
			return !empty($wprecord[$wp_attr]) ? $wprecord[$wp_attr] : $wpmeta[$wp_attr][0];
	} 

	public function mo_sf_sync_decide_value_based_on_conditions($conditions, $wprecord, $wpmeta){
		$user_field_data = array();
		
		for($condition_iterator = 0; $condition_iterator < count($conditions['picklist_wp_fields']); $condition_iterator++) {
			$wp_field = $conditions['picklist_wp_fields'][$condition_iterator];
			$condition = $conditions['picklist_conditions'][$condition_iterator];
			$match = $conditions['picklist_output'][$condition_iterator];
			$result = $conditions['picklist_result'][$condition_iterator];

			if(!isset($user_field_data[$wp_field])) {
				$user_field_data[$wp_field] = $this->mo_sf_sync_read_value_from_wp_data($wp_field, $wprecord, $wpmeta);
			}
			if($this->mo_sf_sync_evaluate_condition(trim($user_field_data[$wp_field]), trim($match), trim($condition))){
				return $result;
			}
		}
		return '';
	}

	public function mo_sf_sync_evaluate_condition($operand1, $operand2, $operator){
		switch($operator) {
			case 'starts-with':
				return strpos($operand1, $operand2) === 0;
			case 'ends-with':
				return ((strrpos($operand1, $operand2) + strlen($operand2)) == strlen($operand1));
			case 'includes':
				return (strpos($operand1, $operand2) !== false);
			case 'must-not-include':
				return (strpos($operand1, $operand2) === false);
		}
		return false;
	}
	
	public function mo_sf_sync_push_to_salesforce($wpid, $wp_object = 'user'){
		$map = $this->mo_sf_sync_get_mapping($wpid, $wp_object);

		if(isset($map['direction']) && $map['direction'] === 'sf_to_wp'){
			$error = new \stdClass();
			$error->message="Mapping Saved is for Salesforce to WordPress Sync";
			return $error;
		}

		if(is_null($map)){
			$error=new \stdClass();
			$error->message="Object and mapping not saved";
			return $error;
		}
		$object_table_structure = $this->ObjectSyncSfToWp->mo_sf_sync_get_wp_db_table_structure($wp_object);
		$meta_fetch_method = $object_table_structure['meta_methods']['read'];
		$sfid = $meta_fetch_method($wpid, "salesforce_".$map["object"]."_ID", true);
		if(!empty($sfid)){
			$response= $this->Salesforce->mo_sf_sync_update_record($sfid, $map["object"], $map["body"]);
			if(is_array($response ) && count($response) && isset($response[0]['errorCode']) && $response[0]['errorCode']==='ENTITY_IS_DELETED'){
				return $this->mo_sf_sync_create_record_in_salesforce($map, $wpid, $object_table_structure, $wp_object);
			}
			if($this->enable_audit_logs) {
				if(is_array($response ) && count($response) && isset($response[0]['errorCode']))
					$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $sfid, $wpid, 'Update', 'Failed', $response[0]['message'], $wp_object);
				else
					$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $sfid, $wpid, 'Update', 'Success', 'Sync Successful with '.$map["object"], $wp_object);
			} 
			else 
				return $response;

		} else {
			return $this->mo_sf_sync_create_record_in_salesforce($map,$wpid,$object_table_structure,$wp_object);
		}
	}

	public function mo_sf_sync_create_record_in_salesforce($map,$wpid,$object_table_structure,$wp_object){
		if(!isset($map)){
			wp_send_json_success(array(
				["message" => "Something went wrong processing the request!"]
			));
			return;
		}
		$response=$this->Salesforce->mo_sf_sync_create_record($map["object"], $map["body"]);
		if(isset($response["id"])){
			$meta_update_method = $object_table_structure['meta_methods']['update'];
			$meta_update_method($wpid,"salesforce_".$map["object"]."_ID",sanitize_text_field($response["id"]));
		}
		if($this->enable_audit_logs) {
			if(isset($response[0]['errorCode']))
				$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, 'Creation Failed', $wpid, 'Create', 'Failed', $response[0]['message'], $wp_object);
			else
				if(isset($response["id"]))
					$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $response["id"], $wpid, 'Create', 'Success', 'Sync Successful with '.$map["object"], $wp_object);
		}
		return $response;
	}
	public function mo_sf_sync_create_composite_request($wpid_list, $sfid_list, $request_content){
		$composite_request = array();
		$i=0;
		foreach($wpid_list as $wpids){
			$sub_request = array();
			$object_name = $request_content[$wpids]['object'];
			if(!empty($sfid_list[$wpids])){
				//update
				$method = 'PATCH';
				$url = '/services/data/' . $this->Salesforce->api_version . '/sobjects/' . $object_name;
				$url .= '/' . $sfid_list[$wpids];
			}else{
				//create
				$method = 'POST';
				$url = '/services/data/' . $this->Salesforce->api_version . '/sobjects/' . $object_name;
			}

			$random_no = strval(wp_rand());
			$refid = 'ref' . $object_name . $random_no;
			$body = $request_content[$wpids]['body'];

			$sub_request['method'] = $method;
			$sub_request['url'] = $url;
			$sub_request['referenceId'] = $refid;
			$sub_request['body'] = $body;

			$composite_request[$i] = $sub_request;
			$i+=1;
		}
		$access_token = $this->Salesforce->access_token;
		$composite_url = $this->Salesforce->instance_url.'/services/data/' . $this->Salesforce->api_version . '/composite';
		$headers = array(
			"Authorization" => "Bearer ".$access_token,
			"Content-Type" => "application/json"
		);
		$composite_request_body = json_encode(array( "compositeRequest" => $composite_request ));

		$request = $this->Salesforce->mo_sf_sync_send_composite_request($composite_url,$headers,$composite_request_body);
		return $request;
		
	}
	public function mo_sf_sync_composite_call_handler($wpid_list, $wp_object = 'user'){		 
		$object_table_structure = $this->ObjectSyncSfToWp->mo_sf_sync_get_wp_db_table_structure($wp_object);
		$meta_fetch_method = $object_table_structure['meta_methods']['read'];
		$composite_request_content = array();		
		foreach($wpid_list as $wpid){
			$map = $this->mo_sf_sync_get_mapping($wpid, $wp_object);
			$sfid_list[$wpid]= $meta_fetch_method($wpid, "salesforce_".$map["object"]."_ID", true);
			$composite_request_content[$wpid] = $map;
			$composite_request_content[$wpid]['nomenclature'] = "salesforce_".$map["object"]."_ID";

		}
		$response = $this->mo_sf_sync_create_composite_request($wpid_list, $sfid_list, $composite_request_content);
		$composite_response_array = $response['compositeResponse'];
		$meta_fetch_method = $object_table_structure['meta_methods']['read'];
		$meta_update_method = $object_table_structure['meta_methods']['update'];
		$i=0;
		foreach($wpid_list as $wpid){
			$map = $this->mo_sf_sync_get_mapping($wpid, $wp_object);
			$update_failed = false;
			$current_response = $composite_response_array[$i]['body'];
			if(isset($current_response["id"])) {
				if(isset($composite_request_content[$wpid]["nomenclature"]) && !empty($composite_request_content[$wpid]["nomenclature"]))
					$meta_update_method($wpid, $composite_request_content[$wpid]["nomenclature"], sanitize_text_field($current_response["id"]));
				else
					$meta_update_method($wpid, 'salesforce_'.$composite_request_content[$wpid]["object"]."_ID", sanitize_text_field($current_response["id"]));
			} else if(is_array($current_response) && count($current_response) && isset($current_response[0]['errorCode'])) {
				if($current_response[0]['errorCode']=='ENTITY_IS_DELETED'){
					$res = $this->mo_sf_sync_create_record_in_salesforce($map,$wpid, $object_table_structure, $wp_object);
					if(isset($res[0]['errorCode']))
						$update_failed = true;
					$current_response = $res;
				}
			}
			if($this->enable_audit_logs) {
				$status_code = $composite_response_array[$i]['httpStatusCode'];
				$sf_obj = $composite_request_content[$wpid]['object'];
				if(is_array($current_response) && count($current_response) && isset($current_response[0]['errorCode']) && $status_code != "204"){
					if($update_failed)
						$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $meta_fetch_method($wpid, $composite_request_content[$wpid]["nomenclature"], true), $wpid, 'Update', 'Failed', $current_response[0]['message'], $wp_object);
					else
						$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, 'Creation Failed', $wpid, 'Create', 'Failed', $current_response[0]['message'], $wp_object);
				} else if($status_code == "204"){
					$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $meta_fetch_method($wpid, $composite_request_content[$wpid]["nomenclature"] , true), $wpid, 'Update', 'Success', 'Sync Successful with '.$sf_obj, $wp_object);

				}
				else{
					$this->audit->mo_sf_sync_add_log(PluginConstants::WPTOSF, $current_response["id"], $wpid, 'Create', 'Success', 'Sync Successful with '.$sf_obj, $wp_object);
				}
			}
			$i+=1;
		}
		
		return "Composite Call Complete";
	}
	public function mo_sf_sync_save_realtime_sync($post_data){
		if(array_key_exists('automatic_user_update',$post_data)){
			$map[PluginConstants::AUTO_USER_UPDATE] = 'on';
			Utils::mo_sf_sync_save_settings(PluginConstants::PROVISION_OBJECT,$map);
		}
		else{
			$map[PluginConstants::AUTO_USER_UPDATE] = 'off';
			Utils::mo_sf_sync_save_settings(PluginConstants::PROVISION_OBJECT,$map);
		}
		Utils::mo_sf_sync_show_success_message('Saved Configuration Successfully !!');
	}
}