<?php

namespace MoSfSyncSalesforce\Handler;

use MoSfSyncSalesforce\Services\DBUtils;
use MoSfSyncSalesforce\Services\AuditDB;
use MoSfSyncSalesforce\Helper\PluginConstants;

class WorkflowIntegrationHandler{

    public function mo_sf_sync_xml_parser($req)
    {		
        $access_key = sanitize_text_field($_REQUEST['accesskey']);
        
        if(empty($access_key))
            $this->mo_sf_sync_xml_response();
        
        $db = DBUtils::instance();

        $mapping_sf_to_wp = $db->mo_sf_sync_get_all_mapping_data();

        if(empty($mapping_sf_to_wp)){
            $this->mo_sf_sync_xml_response();
            return;
        }

        $action = sanitize_text_field($_REQUEST['action']);

        $result = trim( file_get_contents( 'php://input' ) );
        $xml = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $result );

        $parser_create_xml = xml_parser_create();
        xml_parse_into_struct($parser_create_xml, $xml, $vals, $index);
        $i=0;

        while($i<count($vals)){
            if($vals[$i]['tag'] == 'SOBJECT'){
                if(isset($vals[$i]['attributes']['XSI:TYPE'])){
                    $sObject = $vals[$i]['attributes']['XSI:TYPE'];
                }
            }
            $i++;
        }
        if(empty($vals))
            return;
        else
            $xml = simplexml_load_string($xml);
        if(empty($xml))
            return;
        else{
            $json = json_encode($xml);
            $req  = json_decode( $json, true );
            $req['Salesforce Object'] = str_ireplace("sf:","",$sObject);
            $sf_data = array();
            $sf_data['Salesforce Object'] = $req['Salesforce Object'];
        
            if(isset($req['soapenvBody']['notifications']['Notification']['sObject'])){
                foreach($req['soapenvBody']['notifications']['Notification']['sObject'] as $key => $value){
                    $key = substr($key, 2);
                    $sf_data["Object Information"][0][$key] = $value;  
                }
            }
            else{
                $i = count($req['soapenvBody']['notifications']['Notification']);
                $j = 0;
                while($j < $i-1){
                    foreach($req['soapenvBody']['notifications']['Notification'][$j]['sObject'] as $key => $value){
                        $key = substr($key, 2);
                        $sf_data["Object Information"][$j][$key] = $value;  
                    }
                    $j++;
                }
            }

            $this->mo_sf_sync_route_soap_request_processing($sf_data, $mapping_sf_to_wp, $action);
            $this->mo_sf_sync_xml_response();
        }
    }

    public function mo_sf_sync_xml_response(){
        
        $response ="<?xml version='1.0' encoding='UTF-8'?>
        <soapenv:Envelope xmlns:soapenv='http://schemas.xmlsoap.org/soap/envelope/' xmlns:out='http://soap.sforce.com/2005/09/outbound'>
        <soapenv:Header>
        </soapenv:Header>
        <soapenv:Body>
        <out:notificationsResponse>
        <out:Ack>true</out:Ack>
        </out:notificationsResponse>
        </soapenv:Body>
        </soapenv:Envelope>";
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json;charset=utf-8');
        echo $response;
        exit;
    }

    public function mo_sf_sync_route_soap_request_processing($sf_data, $mapping_sf_to_wp, $action){
    
        switch($action){
            case 'store' : 
                $this->mo_sf_sync_store_data_to_wp_objects($sf_data, $mapping_sf_to_wp);
                break;
        }
    }

    function mo_sf_sync_store_data_to_wp_objects($sf_data, $mapping_sf_to_wp){
        $sf_to_wp = new ObjectSyncSfToWp();
        
        $wp_obj_db_struct = $sf_to_wp->mo_sf_sync_get_wp_db_table_structure($mapping_sf_to_wp['wordpress_object']);
        $wp_object_non_meta_columns = $this->mo_sf_sync_get_column_names($wp_obj_db_struct['content_table']);
        foreach($sf_data['Object Information'] as $key => $value)
            $this->mo_sf_sync_save_wp_records(array($value), $mapping_sf_to_wp['wordpress_object'] ,$sf_data['Salesforce Object'], maybe_unserialize($mapping_sf_to_wp['field_mapping'])['field_map'], $wp_obj_db_struct, $wp_object_non_meta_columns);
    }

    private function mo_sf_sync_get_column_names($table_name){
        global $wpdb;
        return $wpdb->get_col("DESC {$table_name}", 0);
    }

    private function mo_sf_sync_save_wp_records($records, $wp_object, $sf_object, $field_map, $object_table_structure, $wp_object_non_meta_columns){
        $enable_audit_logs = get_option(PluginConstants::AUDIT_LOGS);
        $audit = AuditDB::instance();
        foreach($records as $record){
			$wp_upsert_object_id = false;
			$new_wp_object_data = $this->mo_sf_sync_build_data_to_save($field_map, $record);

            $sf_id = $record['Id'];
			
			$wp_object_data = $this->mo_sf_sync_get_record_if_already_synced($sf_object, $sf_id, $wp_object);
			
			$wp_obj_id = false;

			if(!empty($wp_object_data)){
				$wp_obj_id = $wp_object_data[$object_table_structure['id_field']];
				$new_wp_object_data[$object_table_structure['id_field']] = $wp_obj_id;				
				$wp_upsert_object_id = $this->mo_sf_sync_upsert_wp_object($wp_object, $new_wp_object_data, $object_table_structure, $wp_object_non_meta_columns, $wp_obj_id);
			}			
            else{
                $new_wp_object_data['salesforce_'.$sf_object.'_ID'] = $sf_id;
                $wp_upsert_object_id = $this->mo_sf_sync_upsert_wp_object($wp_object, $new_wp_object_data, $object_table_structure, $wp_object_non_meta_columns, false);
            }
            if($enable_audit_logs){
				if(!empty($wp_object_data)){
					if($wp_upsert_object_id)
						$audit->mo_sf_sync_add_log(PluginConstants::SFTOWP, $sf_id, $wp_obj_id, 'Update', 'Success', 'Pull Successful with '.$sf_object, $wp_object);
					else
						$audit->mo_sf_sync_add_log(PluginConstants::SFTOWP, $sf_id, $wp_obj_id, 'Update', 'Failed', 'Pull failed Cannot save data to WordPress object', $wp_object);
				} else {
					if($wp_upsert_object_id)
						$audit->mo_sf_sync_add_log(PluginConstants::SFTOWP, $sf_id, $wp_upsert_object_id, 'Create', 'Success', 'Pull Successful with '.$sf_object, $wp_object);
					else
						$audit->mo_sf_sync_add_log(PluginConstants::SFTOWP, $sf_id, 'Creation Failed', 'Create', 'Failed', 'Pull failed Cannot save data to WordPress object', $wp_object);
				}
			}
		}
		return true;
    }

    private function mo_sf_sync_build_data_to_save($field_map, $sf_record){
        
		$fields = $field_map['field_mapping'];
		$data_type = $field_map['field_type'];
		$field_type = $field_map['type_constraint'];
        
		$new_wp_object_data = array();
        $fieldmap_iterator = 0;
		foreach($fields as $key => $value){
            if($data_type[$fieldmap_iterator] == 'picklist' || $field_type[$fieldmap_iterator] == 'static')
                continue;

			if(!is_array($value) && !empty($sf_record[$key]) )
				$new_wp_object_data[$value] = $sf_record[$key];

            $fieldmap_iterator++;
        }
		return $new_wp_object_data;
	}

    private static function mo_sf_sync_get_record_if_already_synced($sf_object, $sf_id, $wp_object){
		$query_class = 'WP_Query';
		$method = 'get_results';
		$args = array();

		switch($wp_object) {
			case 'user':
				$query_class = 'WP_User_Query';
				break;
			default:
				$args['post_type'] = $wp_object;
				$method = 'get_posts';
		}
		$results = false;
		if (class_exists($query_class)) {
			$args['meta_query'] = array(
				array(
					'key' => 'salesforce_'.$sf_object.'_ID',
					'value' => $sf_id,
				),
			);
			$match_query = new $query_class($args);
			$results = $match_query->$method();

			if(empty($results))
				return false;

			if($wp_object === 'user')
				$results = (array)$results[0]->data;
			else
				$results = (array)$results[0];
		}
		
        return $results;
    }

    private function mo_sf_sync_upsert_wp_object($wp_object, $new_wp_object_data, $object_table_structure, $wp_object_non_meta_columns, $wp_id = false){
		$wp_non_meta_fields = array();
		foreach($new_wp_object_data as $key=>$value){
			if(in_array($key, $wp_object_non_meta_columns)){
				$wp_non_meta_fields[$key] = $value;
				unset($key);
			}
		}
            
		if($object_table_structure['object_name'] == 'post'){
			$wp_non_meta_fields['post_type'] = $wp_object;
			if(empty($wp_id))
				$wp_non_meta_fields['post_status'] = 'publish';
		}
		$method = $object_table_structure['content_methods']['create'];
		if(!empty($wp_id))
			$method = $object_table_structure['content_methods']['update'];
			
			$wp_id = $method($wp_non_meta_fields);

		if(empty($wp_id) || is_wp_error($wp_id)){
			return false;
		}

		$meta_method = $object_table_structure['meta_methods']['update'];
		$this->mo_sf_sync_inset_meta($new_wp_object_data, $wp_id, $meta_method);

		return $wp_id;
	}

    private function mo_sf_sync_inset_meta($meta_array, $wp_obj_id, $method){	
		foreach($meta_array as $key => $value){
			if(strpos($value,"-") != false) {
				$dt = \DateTime::createFromFormat('Y-m-d', $value);
				if($dt && $dt->format('Y-m-d') === $value){
					$value = str_replace('-','',$value);
				}
			} else if(strpos($value,";") != false) {
				$value = array_values(explode(";",$value));
			} 
			$method($wp_obj_id, $key, $value);
		}
		return true;
	}

}
