<?php
namespace MoSfSyncSalesforce\Helper\view;

use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\API\Salesforce;
use MoSfSyncSalesforce\Services\Utils;
use MoSfSyncSalesforce\Handler\ObjectSyncSfToWp;
use MoSfSyncSalesforce\Services\DBUtils;
use MoSfSyncSalesforce\Handler\AjaxHandler;

class FieldMapping
{
	
	use instance;
	private $mapping_data;
	private $object;
	private $field_mapping;
	private $custom_mapping;
	private $sf_objects;
	private $sf_obj_fields;
	public $is_auth_configured;
	public $Salesforce;
	private $wordPress_object;
	private $db;
	private $ajaxhandler;
	private $object_sync_sf_to_wp;
	private $sync_wp_to_sf;
	private $sync_sf_to_wp;

	public static function instance()
	{
		self::$_instance = new self;
		self::$_instance->db = DBUtils::instance();
		self::$_instance->ajaxhandler = new AjaxHandler;
		self::$_instance->mapping_data = self::$_instance->db->mo_sf_sync_get_all_mapping_data();
		self::$_instance->is_auth_configured = Utils::mo_sf_sync_is_authorization_configured();
		self::$_instance->ObjectSyncSfToWp = new ObjectSyncSfToWp();
		return self::$_instance;
	} 

	private function mo_sf_sync_load_mapping_data()
	{
		$this->Salesforce = new Salesforce();
		if (isset($this->mapping_data['salesforce_object'])) {
			$this->object = $this->mapping_data['salesforce_object'];

			if(!array_key_exists('wp_object_select',$this->mapping_data) && !array_key_exists('sync_sf_to_wp',$this->mapping_data) && !array_key_exists('sync_wp_to_sf',$this->mapping_data)){
				$this->field_mapping = isset($this->mapping_data['field_mapping']) ? $this->mapping_data['field_mapping'] : array();
				$this->custom_mapping = isset($this->mapping_data['custom_field_mapping']) ? $this->mapping_data['custom_field_mapping'] : array();
				$this->wordPress_object = 'user';
				$this->sync_wp_to_sf = true;
				$this->sync_sf_to_wp = false;
			}
			else{
				$this->field_mapping = isset($this->mapping_data['field_mapping']) ? maybe_unserialize($this->mapping_data['field_mapping'])['field_map']: array();
				$this->wordPress_object = isset($this->mapping_data['wordpress_object']) ? $this->mapping_data['wordpress_object'] : array();
				$this->sync_wp_to_sf =  ($this->mapping_data['sync_wp_to_sf'] == 1) ? true : false;
				$this->sync_sf_to_wp =  ($this->mapping_data['sync_sf_to_wp'] == 1) ? true : false;
			}	
			if ($this->is_auth_configured){
				$get_object_fields = $this->Salesforce->mo_sf_sync_get_fields($this->mapping_data['salesforce_object']);
			}
		} else {
			$this->field_mapping = array();
			$this->custom_mapping = array();
			$this->object = 'Lead';
			$this->wordPress_object = 'user';
			$this->sync_wp_to_sf = true;
			$this->sync_sf_to_wp = false;
			if ($this->is_auth_configured){
				$get_object_fields = Utils::mo_sf_sync_get_transient('transient_lead_object');
			}
		}

		if ($this->is_auth_configured) {
			$res = get_transient('transient_get_object');
			if(empty($res)){
				$res = $this->Salesforce->mo_sf_sync_get_objects();
				if ( ! is_array( $res ) && $res == 'Please install miniOrange on your connected salesforce environment' ) {
					Utils::mo_sf_sync_information_message( 'Please install miniOrange on your connected salesforce environment', 'alert' );
				} elseif ( is_array( $res ) && is_string( $res[0] ) && stripos( $res[0], 'cURL error' ) === 0 ) {
					Utils::mo_sf_sync_information_message( 'Unable to connect to Salesforce please refresh and try again', 'alert' );
				}
			}

			$this->sf_objects = $res;
			$this->sf_obj_fields = $get_object_fields["fields"] ?? array();
		} else {
			$this->sf_objects = array();
			$this->sf_obj_fields = array();
		}
	}

	function mo_sf_sync_multiple_object_menu()
	{
		if (isset($_GET['mapping_label']) && !isset($_GET['action'])) {
			$this->mo_sf_sync_object_display();
		} else {
			$this->mo_sf_sync_show_object_list($this->mapping_data);
			$this->mo_sf_sync_display_test_config();
		}
	}

	function mo_sf_sync_show_object_list($mapping_list)
	{
		$is_mapping_configured = (isset($mapping_list) && !empty($mapping_list));
		$mapping_direction = ($is_mapping_configured && $mapping_list['sync_wp_to_sf'] == 1) ? "WordPress to Salesforce" : "Salesforce to WordPress";
		?>
		<input type="hidden" id="mo_sf_sync_home_url" value="<?php echo home_url() ?>">
		<div class="mo-sf-sync-tab-content-tile mo-sf-mt-4">
			<h1 class="mo-sf-form-head">Configure Attribute Mapping</h1>

			<div class="mo-sf-sync-tab-content-tile-content">

				<table class="mo-sf-sync-object-list">
					<h3 class="mo-sf-form-head mo-sf-form-head-bar mo-sf-mt-5">Object Mapping List:</h3>
					<thead>
						<tr>
							<td width="20%">Mapping Label</td>
							<td width="45%">Sync Direction</td>
							<td width="18%" style="text-align:center">Edit Mapping</td>
							<td style="text-align:center">Delete Mapping</td>
						</tr>
					</thead>
					<?php if ($is_mapping_configured) {
					?>
						<tr>
							<td><?php echo esc_attr($mapping_list['label']) ?></td>
							<td><?php echo esc_attr($mapping_direction) ?></td>
							<td class="mo-sf-sync-object-list-edit-button">
								<a <?php if($this->is_auth_configured) echo 'href="?page=mo_sf_sync&tab=manage_users&mapping_label='.esc_attr($mapping_list['label']).'"' ?>>Edit</a>
							</td>
							<td>
								<form method="post" id="delete_config" name="delete_config" action="" enctype="multipart/form-data">
									<?php wp_nonce_field('mo_sf_sync_delete'); ?>
									<center>
										<label>
											<input type="hidden" name="option" value="mo_sf_sync_delete" />
											<input type="hidden" name="mapping_label" value="<?php echo $mapping_list['label']; ?>">
											<p></p>
											<input type="submit"  class="mo-sf-sync-object-list-delete-button" name="button" value="Delete" onclick="return confirm('Do you really want to delete the mapping ?')" />
										</label>
									</center>
								</form>
							</td>
						</tr>
					<?php
					}
					?>
					</tbody>
				</table>
				<div>
					<?php
					$message = (!$this->is_auth_configured) ? 'Authorization not configured' : 'Available with premium version of the plugin';
					if (!$is_mapping_configured && $this->is_auth_configured) { ?>
						<div style="font-size:medium;margin:10px 0px 10px 0px"><b>No object mapping configured!</b><br></div>
						<div class="mo-sf-text-center">
							<a href="?page=mo_sf_sync&tab=manage_users&mapping_label=">
								<button class="mo-sf-btn-cstm">Add Object Mapping</button>
							</a>
						</div>
					<?php } else { ?>
						<p></p>
						<div class="mo-sf-sync-tab-content-tile-content mo-sf-prem-info">
							<div style="text-align:center;">
								<button style="height:40px;" style="position: relative;" disabled class="mo-sf-btn-cstm">Add Object mapping</button>
							</div>
							
							<div class="mo-sf-prem-lock" style="margin-top:-17px">
								<img style="height: 40px;margin-top:-50px" src="<?php echo esc_url(plugin_dir_url(__FILE__) . '../../images/crown.png'); ?>" alt="">
								<?php
								
								if(!$this->is_auth_configured) 
									echo '<p class="mo-sf-prem-text">Please Configure & Authorize the Salesforce Connected Application in the <a href="?page=mo_sf_sync&tab=app_config" class="mo-sf-text-warning">Manage Applications</a> tab</p>';
								else
									echo '<p class="mo-sf-prem-text">Available in premium plugin. <a href="?page=mo_sf_sync&tab=licenseplan" class="mo-sf-text-warning">Click here to upgrade</a></p>'; 
								?>
							</div>							
						</div>
							
					<?php } ?>
				</div>
			</div>
		</div>
		<?php
	}

	public function mo_sf_sync_object_display()
	{
		$this->mo_sf_sync_load_mapping_data();
		$this->mo_sf_sync_mapping_config();
	}

	private function mo_sf_sync_mapping_config(){
		
		$wordpress_types_not_posts_include = array('user', 'post', 'comment', 'category', 'tag');
        $post_types = get_post_types();
		unset($post_types['post']);
		$wp_object_types = array_merge($wordpress_types_not_posts_include,$post_types);
		$wp_object_types_allowed = array('user','post');		
		$data_fields = $this->ObjectSyncSfToWp->mo_sf_sync_get_wp_obj_fields($this->wordPress_object);
		$is_mapping_configured = (isset($this->mapping_data) && !empty($this->mapping_data));
		$wp_object_change_message = ($is_mapping_configured)?'':'hidden';
		?>
		<form method="post" action="<?php echo admin_url(). 'admin.php?page=mo_sf_sync&tab=manage_users' ; ?>">
			<input type="hidden" name="option" value="mo_sf_sync_client_object">
			<input type="hidden" name="nonce_" value="<?php echo wp_create_nonce('mo_sf_sync_client_object'); ?>">
			<input type="hidden" id="saved_mapping" value='<?php echo (isset($this->field_mapping['name_constraint'])) ? json_encode($this->field_mapping['name_constraint']) : '' ?>'> 

			<section class="accordion mo-sf-mt-4" >
				<input type="checkbox" name="collapse" checked="checked" >
				<h2 class="handle">
					<label class = "mo-sf-sync-coll-div-head" ><b>1. Select WordPress Object</b></label>
				</h2>
				<div class="mo-sf-dflex mo-sf-sync-tab-content-tile content" style="border: none;box-shadow:none">
					<div class=" mo-sf-col-md-3">
						<b><span>WordPress object <span class="mo-sf-text-danger">*</span></b></span>
					</div>
					<div class="mo-sf-col-md-8-field-mapping mo-sf-dflex">
						<div>
							<?php
								if($is_mapping_configured){
									?>
										<input type="text" class="wp_object_select" id="wp_object_select" style="width: 300px;" name="wp_object_select" readonly value="<?php echo esc_attr( $this->wordPress_object)?>" >
									<?php	
								} else {
							?>
							<select class="form-control wp_object_select" id="wp_object_select" style="width: 300px;" name="wp_object_select" data-wp-object="<?php echo esc_attr( $this->wordPress_object) ?>" >
								<?php	
									
									foreach($wp_object_types as $value){
										$select_wp_obj = $value == $this->wordPress_object ? "selected" : "";
										$disabled_wp_obj = in_array($value, $wp_object_types_allowed) ? "" : "disabled";
										$premium_ad = '';
										if(!empty($disabled_wp_obj)){
											$premium_ad = ' [Available in Premium]';
										}
										echo "<option value='";
										echo esc_attr($value);
										echo "'";
										echo esc_attr($select_wp_obj) .' '. esc_attr($disabled_wp_obj);
										echo ">".$value.$premium_ad."</option>";
									}
								?>
							</select>
							<?php } ?>
							<br/><br/>
							<div class="mo_sf_sync_help_desc" <?php echo $wp_object_change_message; ?>>To change the WordPress Object please delete the existing mapping and create a new one.</div>
						</div>
						<div style="margin-left:100px">Support for Custom post types and additional WordPress objects and Integrations available in Premium version<br> <a href='?page=mo_sf_sync&tab=demo_setup'><b>Try Now For Free</b></a>
						</div>
					</div>
				</div>
			</section>

			<section class="accordion mo-sf-mt-4" >
				<input type="checkbox" name="collapse" checked="checked">
				<h2 class="handle">
					<label class = "mo-sf-sync-coll-div-head"><b>2. Select Sync Direction</b></label>
				</h2>
				<div class="mo-sf-sync-tab-content-tile content" style="box-shadow:none">
					<div class="mo-sf-dflex" style="border: none;box-shadow:none;">
						<div class=" mo-sf-col-md-3">
							<b><span>Choose Sync Direction <span class="mo-sf-text-danger">*</span></span></b>
						</div>
						<div class="mo-sf-col-md-8-field-mapping">
							<span class="mo-sf-col-md-5" style="background-color: white;">
								<input type="radio" name="sync_wp_to_sf" id="sync_wp_to_sf" <?php checked($this->sync_wp_to_sf, true); ?> onchange="mo_sf_sync_change_direction('sync_wp_to_sf')">
								<span>Sync From WordPress to Salesforce</span>
							</span>
							<span class="mo-sf-col-md-5 mo-sf-ml" style="background-color:white;">
								<input type="radio" name="sync_sf_to_wp" id="sync_sf_to_wp" <?php checked($this->sync_sf_to_wp, true); ?> onchange="mo_sf_sync_change_direction('sync_sf_to_wp')">
								<span>Sync From Salesforce to WordPress</span>
							</span>
						</div>
					</div>
				</div>
			</section>

			<section class="accordion mo-sf-mt-4" >
				<input type="checkbox" name="collapse" checked="checked">
				<h2 class="handle">
					<label class = "mo-sf-sync-coll-div-head" ><b>3. Select Salesforce Object</b></label>
				</h2>
				<div class="mo-sf-dflex mo-sf-sync-tab-content-tile content" style="border: none;box-shadow: none">
					<div class=" mo-sf-col-md-3">
						<b><span>Salesforce object <span class="mo-sf-text-danger">*</span></span></b>
					</div>
					<div class="mo-sf-col-md-8-field-mapping">
						<div>
							<select class="form-control" id="object_select" style="width: 300px;" name="object_select" data-object="<?php echo esc_attr($this->object) ?>">
								<option value="">Select a Salesforce Object</option>
								<?php
								foreach ($this->sf_objects as $obj) {
									echo "<option value=" . esc_attr($obj['name']);
									$_temp = $obj['name'] == $this->object ? " selected" : "";
									echo esc_attr($_temp);
									echo ">" . esc_attr($obj['label']) . "</option>";
								}
								?>
							</select>
						</div>
					</div>
				</div>
			</section>

			<div style="text-align:center" id="sf-obj-search" hidden>
				<div class="" style="display: flex;align-items: center;justify-content: center;">
					<img  alt="Loading..." src="<?php echo esc_url(plugin_dir_url(__FILE__) .'../../images/preloder1.gif') ?>" />
					<span style="font-style: italic;margin-left: -30px;">Loading... Please Wait...</span></div>
				</div>
			
			<section class="accordion field_mapping mo-sf-mt-4" hidden>
				<input type="checkbox" name="collapse" checked="checked" >
				<h2 class="handle">
					<label class = "mo-sf-sync-coll-div-head" ><b>4. Map WordPress fields to Salesforce fields</b></label>
				</h2>
				<div class="mo-sf-sync-tab-content-tile mo-sf-dflex content" style="box-shadow: none;">
					<div class="mo-sf-col-md-3">
						<b><span>Field Mapping</span></b><span style="margin-top:2px;margin-left:2px" class="dashicons dashicons-editor-help information">
							<p class="information-text">Create a map between all the Salesforce and WordPress fields you require to sync. </p>
						</span>
					</div>
					<div class="sf-wp-mapping-div mo-sf-col-md-8-field-mapping">
						<?php 			
						if(isset($this->field_mapping['name_constraint']))
							foreach($this->field_mapping['name_constraint'] as $key => $value){
								?>
								<section class="accordion">
									<input type="checkbox" name="collapse" id="<?php echo $value ?>" checked="checked">
									<input type="hidden" name="sf_fields[]" value="<?php echo $value ?>">
									<input type="hidden" name="name_constraint[]" value="<?php echo $value ?>">
									<input type="hidden" name="type_constraint[]" value="<?php echo $this->field_mapping['type_constraint'][$key] ?>">
									<input type="hidden" name="maxlength_constraint[]" value="<?php echo $this->field_mapping['size_constraint'][$key] ?>">
									<h2 class="handle">
										<label class = "mo-sf-sync-coll-div-head" for="<?php echo $value ?>"><b><?php echo $value; if($this->mo_sf_sync_check_required_field($value) == true) echo '<span style="font-size: 14px !important; color:red">*</span>' ?></b><span style="font-size: 14px !important;font-style: italic;">  [Name:<?php echo $value ?>, Type: <?php echo $this->field_mapping['type_constraint'][$key] ?>, Max-length: <?php echo $this->field_mapping['size_constraint'][$key] ?>] </span>
										<button style="float:right ;border:none;cursor:pointer;background:none" onclick="deleteAttr(event);return false;"><span class="dashicons dashicons-trash"></button>
										</label>
									</h2>
									<div class="mo-sf-dflex content" style="border-bottom: 0;">
										<div class=" mo-sf-col-md-3" style="border-bottom:none">
											<span>Field Type</span>
										</div>
										<div class="mo-sf-col-md-8-field-mapping">
											<div>
												<select class="form-control" id="field_type_<?php echo $value ?>" style="width: 300px;" name="field_types[]" onchange="mo_sf_sync_change_field_type('<?php echo $value ?>')">
													<option <?php if($this->field_mapping['field_type'][$key] == "standard") echo'selected'; else echo ''; ?> value="standard">
													<?php 
														if($this->field_mapping['type_constraint'][$key] == 'picklist') 
															echo 'Standard fields';
														else
															echo 'Standard WordPress fields'; ?>
													</option> 
													<option <?php if($this->field_mapping['field_type'][$key] == "static") echo'selected'; else echo ''; ?> value="static">
													<?php 
														if($this->field_mapping['type_constraint'][$key] == 'picklist') 
															echo 'Conditional fields';
														else
															echo 'Static'; ?>
													</option> 
												</select>
											</div>
										</div>
									</div>
									<div class="mo-sf-dflex content" style="margin-top:-20px !important">
										<div class="mo-sf-col-md-3">
											<span>Select Field</b></span>
										</div>
										<?php
										 if($this->field_mapping['type_constraint'][$key] == 'picklist') {
										if($this->field_mapping['field_type'][$key] == "static"){
										?>
												<div class="mo-sf-col-md-8-field-mapping">
													<div id="sync_sel_fields_div_<?php echo $value; ?>">
														<div id="picklist_conditon_table_<?php echo $value; ?>">
															<?php $i = 0;
															foreach($this->field_mapping['field_mapping'][$value]['picklist_wp_fields'] as $left => $right ){
																?>
																<?php if($i>0) echo '<div style="text-align:center"><div class="mo_sf_sync_conditional_or">OR</div></div>'; $i++; ?>
																<div class="mo-sf-dflex">
																	<div>
																		<select class="form-control" name="wordpress_fields[<?php echo $key ?>][picklist_wp_fields][]" style="width:115px">
																			<?php
																			foreach($data_fields as $wpfield=>$key_array){
																				echo "<option ";
																				$selected = $key_array['key']==$right? " selected":"";
																				echo esc_attr($selected);
																				echo ">".esc_attr($key_array['key'])."</option>";
																			} 
																			?>
																		</select>
																	</div>
																	<div><span>Must</span></div>
																	<div>
																		<select class="form-control operator" name="wordpress_fields[<?php echo $key ?>][picklist_conditions][]" style="width:115px" >
																			<option  value='starts-with' <?php if($this->field_mapping['field_mapping'][$value]['picklist_conditions'][$left] == "starts-with") echo'selected'; ?> >Starts with</option>
																			<option  value='ends-with' <?php if($this->field_mapping['field_mapping'][$value]['picklist_conditions'][$left] == "ends-with") echo'selected'; ?> >Ends With</option>
																			<option  value='includes' <?php if($this->field_mapping['field_mapping'][$value]['picklist_conditions'][$left] == "includes") echo'selected'; ?> >Includes</option>
																			<option  value='must-not-include' <?php if($this->field_mapping['field_mapping'][$value]['picklist_conditions'][$left] == "must-not-include") echo'selected'; ?> >Must Not Include</option>
																		</select>
																	</div>
																	<div ><span>the value</span></div>
																	<div>
																		<input type="text" readonly class="form-control" name="wordpress_fields[<?php echo $key ?>][picklist_output][]" style="width:115px" value="<?php echo $this->field_mapping['field_mapping'][$value]['picklist_output'][$left]  ?> ">
																	</div>
																	<div >then value synced will be</div>
																	<div>
																		<input type="text" readonly class="form-control" name="wordpress_fields[<?php echo $key ?>][picklist_result][]" style="width:115px" value="<?php echo $this->field_mapping['field_mapping'][$value]['picklist_result'][$left]  ?> ">
																	</div>
																	
																</div>									
																<?php
															}?>
														</div>   
													</div>
											</div>
										</div>
										<?php
									}
									else{
										?>
										<div class="mo-sf-col-md-8-field-mapping">
												<div id="sync_sel_fields_div_<?php echo $value; ?>">
												<input type="text" class="form-control" name="wordpress_fields[]" style="width:300px" value="<?php echo $this->field_mapping['field_mapping'][$value]?>">
												</div>
											</div>
										</div>
										<?php
									}
								}else
									{?>
									<div class="mo-sf-col-md-8-field-mapping">
										<div id="sync_sel_fields_div_<?php echo $value;?>">
									<?php
										if($this->field_mapping['field_type'][$key] != "static") {?>
												<select class="form-control select2" style="width: 300px;" name="wordpress_fields[]">
													<?php
													foreach($data_fields as $wpfield=>$key_array){
														echo "<option ";
														$selected = $key_array['key']==$this->field_mapping['field_mapping'][$value]? "selected":"";
														echo esc_attr($selected);
														echo ">".esc_attr($key_array['key'])."</option>";
													} 
												?>
												</select>
										<?php
										}
										else{
											?>
											<input type="text" style="width: 300px;" name="wordpress_fields[]" value="<?php echo $this->field_mapping['field_mapping'][$value]; ?>">
											<?php
										}
										?>
										</div>
									</div>
									</div>   
									<?php	
									}
									?>             
								</section>     
								<?php
							}
							?>
					</div>
				</div>
			</section>
			</div>
			<div id="save_obj_map_btn" style="text-align: center;" hidden>
				<input type="submit" class="mo-sf-btn-cstm"  value="Save Object Mapping" >
			</div>
		</form>
		<?php
		if (!isset($this->mapping_data) || empty($this->mapping_data)) {
		?>
			<input type="hidden" id="set_field_map" value="field_map_is_not_set">
		<?php
		}
	}

	private function mo_sf_sync_check_required_field($selected_field){
		foreach ($this->sf_obj_fields as $field)
			if (($selected_field == $field["name"]) && ($field["createable"] && !$field["nillable"] && !$field["defaultedOnCreate"] && $field["type"] !== "boolean"))
				return true;
	}
	
	public function mo_sf_sync_create_fields($selected_field)
	{
		$disabled = ' ';
		foreach ($this->sf_obj_fields as $field) {
			if(in_array($field["name"],$this->field_mapping['name_constraint'])){
				$disabled = 'disabled';
			}
			$required = "";
			if ($field["createable"] && !$field["nillable"] && !$field["defaultedOnCreate"] && $field["type"] !== "boolean")
				$required = "* ";
			echo '<option value="'.esc_attr($field["name"]).','.$field['type'].','.$field['length'].'"'.selected($field['name'], $selected_field, false) . ' '.$disabled.'>' . $required . esc_attr($field["label"]) . ' (' . esc_attr($field['type']) . ')</option>';
		}
	}
	
	public function mo_sf_sync_display_test_config()
	{
		if(empty($this->mapping_data))
			return;
		if($this->mapping_data['wordpress_object'] != 'user' && $this->mapping_data['wordpress_object'] !='post')
			return;
		$is_mapping_configured = (isset($this->mapping_data['label']) && !empty($this->mapping_data['label']) ? true : false);
		if(array_key_exists('sync_sf_to_wp',$this->mapping_data) && $this->mapping_data['sync_sf_to_wp'] == '1' ){
			$outbound_url = $this->db->mo_sf_sync_get_data_from_metatable($this->mapping_data['id'],'outbound_redirect_uri')['outbound_redirect_uri'];
			?>
			</br>
				<div class="mo-sf-sync-tab-content-tile" id="test-config-tile" 
				<?php 
					if(Utils::mo_sf_sync_is_authorization_configured() && $is_mapping_configured)
						echo '';
					else
						echo 'hidden';
				?>
				>
					<h2 class="mo-sf-form-head mo-sf-form-head-bar mo-sf-mt-4">Real Time Salesforce to WordPress Sync</h2>
					<div class="mo-sf-sync-tab-content-tile-content">
						<div class="mo_sf_sync_help_desc">
						<h4>
							<span>
								<b><a class="mo-sf-sync-guide-button" target="blank" href="https://plugins.miniorange.com/salesforce-wordpress-real-time-sync-workflow-automation">Click Here</a></b>
								to open an extensive<b> step by step guide </b> to configure <b>Real Time Salesforce to WordPress Sync</b>.
							</span>
						</div>
						</br>
						<div class="mo-sf-dflex mo-sf-ml-1 mo-sf-mt-8">
							<div class="mo-sf-col-md-4" >
								<h2>Redirect URI for outbound message: </h2>
							</div>
							<div class="mo-sf-dflex mo-sf-ml-1">
								<div class="mo_sf_sync_help_desc" style="word-break: break-all;">
									<code id="ob_redirect_url" style="background: none;"><b><?php echo $outbound_url ?></b></code>
								</div>
								
								<div style="margin-left: 30px ;">
									<i  class="mo_copy copytooltip rounded-circle" onclick="mo_sf_sync_copyToClipboard(this, '#ob_redirect_url', '#metadata_url_copy');"><span class="dashicons dashicons-admin-page"></span><span id="ob_redirect_url_copy" class="copytooltiptext">Copy to Clipboard</span></i>
								</div>
							</div>
						</div>
					</div>
				</div>
			<?php
		}
		else{
			?>
			</br>
				<div class="mo-sf-sync-tab-content-tile" id="test-config-tile" 
				<?php 
				if(Utils::mo_sf_sync_is_authorization_configured() && $is_mapping_configured)
					echo '';
				else
					echo 'hidden';
				?>
				>
					<h2 class="mo-sf-form-head mo-sf-form-head-bar mo-sf-mt-4">Test Connection</h2>
					<div class="mo-sf-sync-tab-content-tile-content">
						<table class="mo-sf-sync-tab-content-app-config-table">
							<tr>
								<td colspan="2" class="mo-sf-note">
									<h4> Select a WordPress user to test connection. This will create a record in the Salesforce as a selected object and configured field mapping.</h4>
									<?php if (!$is_mapping_configured) { ?>
										<h4 style="color:red">Please configure field mapping first before testing the connection!</h4>
									<?php } ?>
								</td>
							</tr>
							<tr>
								<td><br></td>
							</tr>
							<tr>
								<td class="left-div">
									<span>Search Users</span>
								</td>
								<td class="mo-sf-dflex">
									<input type="text" class="mo-sf-w-3" name="query" id="enter_search" placeholder="Enter Username/User Email to search user." />
									<input type="button" id="search_users" class="mo-sf-ml-5 mo-sf-btn-cstm" value="Search">
									<span class="mo-sf-sync-user-search-alert" id="push_plh" name="push_plh" type="text" hidden>No User Found </span>
								</td>
							</tr>
							<tr>
								<td class="left-div">
									<span>Users</span>
								</td>
								<td class="right-div mo-sf-dflex">
									<select class="form-control mo-sf-w-3" id="upn_id" name="upn_id">
									</select>
									<input id="push_attributes" type="button" class="mo-sf-ml-5 mo-sf-btn-cstm" <?php if (!$is_mapping_configured) echo 'disabled'; ?> value="Push" style="width:100px">
								</td>
							</tr>
						</table>
					</div>
				</div>
			<?php
		}
		
	}

}