<?php
namespace MoSfSyncSalesforce\Services;
use MoSfSyncSalesforce\Helper\Instance;
require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
class AuditDB
{
    use Instance;
    public $audit_log_table_name;
    public $db;
    public static function instance()
	{
        global $wpdb;
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self;
		}
        self::$_instance->db = $wpdb;
        self::$_instance->audit_log_table_name = "mo_sf_sync_audit_log_db";
		return self::$_instance;
	}
 
    public function mo_sf_sync_create_audit_log_table() {

        $current_charset_collate = $this->db->get_charset_collate();
        $create_table_query = "CREATE TABLE $this->audit_log_table_name (
            id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
            direction text COLLATE utf8mb4_unicode_ci NOT NULL,
            salesforce_id text COLLATE utf8mb4_unicode_ci NOT NULL,
            wordpress_object text COLLATE utf8mb4_unicode_ci NOT NULL,
            wordpress_id text COLLATE utf8mb4_unicode_ci NOT NULL,  
            user_action text COLLATE utf8mb4_unicode_ci NOT NULL,
            action_status text COLLATE utf8mb4_unicode_ci NOT NULL,
            response text COLLATE utf8mb4_unicode_ci NOT NULL,
            time_stamp timestamp NOT NULL DEFAULT current_timestamp() ON UPDATE current_timestamp(),
            PRIMARY KEY (id)
          ) ENGINE=InnoDB $current_charset_collate";

        $created_table = dbDelta($create_table_query,true);   

        if(empty($created_table))
            return false;
        return true;
    }

    public function mo_sf_sync_get_log_count($action_status='ALL'){
        $select_query = "SELECT count(id) as record_count FROM $this->audit_log_table_name";

        if($action_status !== 'ALL')
            $select_query .= " WHERE action_status='$action_status'";

        return $this->db->get_results($select_query);
    }


    public function mo_sf_sync_get_latest_log_timestamp(){
        $select_query = "SELECT time_stamp FROM $this->audit_log_table_name ORDER BY id DESC LIMIT 1";
        return $this->db->get_results($select_query);
    }

    public function mo_sf_sync_get_transaction($transaction_id){
        $select_query = "SELECT * FROM $this->audit_log_table_name WHERE id=$transaction_id";
        return $this->db->get_results($select_query);
    }

	public function mo_sf_sync_add_log($direction="",$salesforce_id="",$username="",$user_action="",$action_status="",$response="", $wp_object = 'user') {
        $data = array(
            'direction'         => $direction,
            'salesforce_id'     => $salesforce_id,
            'wordpress_id'      => $username,
            'user_action'       => $user_action,
            'action_status'     => $action_status,
            'response'          => $response,
            'wordpress_object'  => $wp_object
        );
        $format=array('%s','%s','%s','%s','%s','%s','%s');
        $this->db->insert($this->audit_log_table_name, $data, $format);
    }

    public function mo_sf_sync_get_audit_logs() {
        $select_query = "SELECT * FROM $this->audit_log_table_name";
        $query_result = $this->db->get_results($select_query);
        
        return $query_result;
    }

    public function mo_sf_sync_clear_all_logs() {
        $this->db->query("TRUNCATE TABLE `$this->audit_log_table_name`");
    }

   function mo_sf_sync_save_advance_search_settings(){
        if ($this->mo_sf_sync_check_option_admin_referer('mo_sf_sync_advanced_reports')) {
        
            $username = "";
            $direction = "";
            $status = "";
            $user_action = "";
            $from_date = "";
            $to_date = "";
            if (isset($_POST['username']))
                $username = sanitize_text_field($_POST['username']);

            if (isset($_POST['direction']))
                $direction = sanitize_text_field($_POST['direction']);

            if (isset($_POST['status']))
                $status = sanitize_text_field($_POST['status']);

            if (isset($_POST['user_action']))
                $user_action = sanitize_text_field($_POST['user_action']);

            if (isset($_POST['from_date']))
                $from_date = sanitize_text_field($_POST['from_date']);

            if (isset($_POST['to_date']))
                $to_date = sanitize_text_field($_POST['to_date']);

            update_option('mo_sf_sync_advanced_search_username', $username);
            update_option('mo_sf_sync_advanced_search_direction', $direction);
            update_option('mo_sf_sync_advanced_search_status', $status);
            update_option('mo_sf_sync_advanced_search_action', $user_action);
            update_option('mo_sf_sync_advanced_search_from_date', $from_date);
            update_option('mo_sf_sync_advanced_search_to_date', $to_date);
            update_option('mo_sf_sync_advanced_reports', true);
        }
        else if( $this->mo_sf_sync_check_option_admin_referer('mo_sf_sync_clear_advance_search' ) ) {
            delete_option('mo_sf_sync_advanced_search_username');
            delete_option('mo_sf_sync_advanced_search_direction');
            delete_option('mo_sf_sync_advanced_search_status');
            delete_option('mo_sf_sync_advanced_search_action');
            delete_option('mo_sf_sync_advanced_search_from_date');
            delete_option('mo_sf_sync_advanced_search_to_date');
            delete_option('mo_sf_sync_advanced_reports');
        }
    }

    function mo_sf_sync_check_option_admin_referer( $option_name ) {
        return ( isset( $_POST['option'] ) and $_POST['option'] == $option_name and check_admin_referer( $option_name ) );
    }

	public function mo_sf_sync_get_audit_using_advanced_search() {
        $myrows = "";
        if (get_option('mo_sf_sync_advanced_reports')) {
            
            $username = get_option('mo_sf_sync_advanced_search_username');
            $direction = get_option('mo_sf_sync_advanced_search_direction');
            $status = get_option('mo_sf_sync_advanced_search_status');
            $user_action = get_option('mo_sf_sync_advanced_search_action');
            $from_date = get_option('mo_sf_sync_advanced_search_from_date');
            $to_date = get_option('mo_sf_sync_advanced_search_to_date');
           

            $where_clause = " where ";
            $is_previous_added = false;
            if ($username) {
                $where_clause .= " wordpress_id LIKE '".$username."%'";
                $is_previous_added = true;
            }

            if ($direction && $direction != "default" ) {
                if ($is_previous_added) {
                    if ($direction == "sync wp to sf")
                        $where_clause .= " AND direction != 'sync sf to wp'";
                    else
                        $where_clause .= " AND direction = '" . $direction . "'";
                } else {
                    if ($direction == "sync wp to sf")
                        $where_clause .= " direction != 'sync sf to wp'";
                    else
                        $where_clause .= " direction = '" . $direction . "'";

                    $is_previous_added = true;
                }
            }

            if ($status && $status != "default") {
                if ($is_previous_added) {
                    if ($status == "failed") {
                        $where_clause .= " AND action_status != 'success'";
                    } else {
                        $where_clause .= " AND action_status = '" . $status . "'";
                    }
                }
                else {
                    if ($status == "failed") {
                        $where_clause .= " action_status != 'success'";
                    } else {
                        $where_clause .= " action_status = '" . $status . "'";
                    }
                    $is_previous_added = true;
                }
            }

            if ($user_action && $user_action != "default")
            {
               if ($is_previous_added)
               {
                   if ($user_action == "Create") {
                       $where_clause .= " AND user_action != 'Update'";
                   } else {
                       $where_clause .= " AND user_action = '" . $user_action . "'";
                   }
               }
               else 
               {
                   if ($user_action == "Create") {
                       $where_clause .= " user_action != 'Update'";
                   } else {
                       $where_clause .= " user_action = '" .$user_action . "'";
                   }
                   $is_previous_added = true;
               }
           }
            $has_date_error = false;
            if ($from_date && $to_date && $from_date != $to_date) {
                $frm_date = \DateTime::createFromFormat('Y-m-d', $from_date);
                $t_date = \DateTime::createFromFormat('Y-m-d', $to_date);
                if ($frm_date->getTimestamp() > $t_date->getTimestamp()) {
                    update_site_option( 'mo_sf_sync_message', 'Invalid selection date interval');
                    $has_date_error = true;
                } else
                    $where_clause .= "AND time_stamp BETWEEN ". "'$from_date 00:00:00 '"." AND ". "'$to_date 00:00:00 '";
            } 
            
            if ($has_date_error) 
                $myrows = $this->db->get_results("SELECT * FROM " . $this->audit_log_table_name . " order by id desc limit 5000");
            else {
                if($where_clause == " where ")
                    $where_clause = "";
                $myrows = $this->db->get_results("SELECT * FROM " . $this->audit_log_table_name . $where_clause);
            }
        }
        else
            $myrows = $this->db->get_results("SELECT * FROM " . $this->audit_log_table_name . " order by id desc limit 5000");
        
        return $myrows;
}
}