<?php
namespace MoSfSyncSalesforce\Services;

use MoSfSyncSalesforce\Helper\Instance;

class DBUtils{
    use Instance;
    public $db;
    public $table_name;
	public $meta_table_name;

	public static function instance() {
		global $wpdb;
		self::$_instance                  = new self;
		self::$_instance->db              = $wpdb;
		self::$_instance->table_name      = 'mo_sf_sync_object_mapping';
		self::$_instance->meta_table_name = 'mo_sf_sync_object_mapping_meta';
		return self::$_instance;
	}

    public function mo_sf_sync_create_mapping_tables(){
        $current_charset_collate = $this->db->get_charset_collate();
        $create_table_query = "CREATE TABLE IF NOT EXISTS $this->table_name(
            id bigint(20) NOT NULL AUTO_INCREMENT,
            label varchar(64) NOT NULL,
            salesforce_object varchar(255) NOT NULL,
            wordpress_object varchar(128) NOT NULL,
            sync_sf_to_wp tinyint(1) NOT NULL,
            sync_wp_to_sf tinyint(1) NOT NULL,
            PRIMARY KEY (id)
          )ENGINE=InnoDB $current_charset_collate";

        $this->db->get_results($create_table_query);

        $create_meta_table_query = "CREATE TABLE IF NOT EXISTS $this->meta_table_name (
            id tinyint(4) NOT NULL AUTO_INCREMENT,
            mapping_id bigint(20) NOT NULL,
            meta_key longtext NOT NULL,
            meta_value longtext NOT NULL,
            PRIMARY KEY (id),
            FOREIGN KEY (mapping_id) REFERENCES $this->table_name(id)
          )"; 
          $this->db->get_results($create_meta_table_query);
          return;
    }

    public function mo_sf_sync_save_mapping_in_object_table($sf_object,$wp_object,$direction,$field_map){
        if($direction == 'sync_sf_to_wp'){
            $sync_sf_to_wp = 1;
            $sync_wp_to_sf = 0;
        }
        else{
            $sync_sf_to_wp = 0;
            $sync_wp_to_sf = 1;
        }
        $data = array(
            'label' => $sf_object.'_'.$wp_object.'_Map',
            'salesforce_object' => $sf_object,
            'wordpress_object' => $wp_object,
            'sync_sf_to_wp' => $sync_sf_to_wp,
            'sync_wp_to_sf' => $sync_wp_to_sf
        );
        $format = array('%s','%s','%s','%d','%d');
        $mapping_id = $this->mo_sf_sync_if_mapping_already_exist();

        if($sync_sf_to_wp){
            $outbound_redirect_uri = home_url().'/?method=soap&action=store&mappinglabel='.urlencode($data['label']) .'&accesskey='.$this->mo_sf_sync_create_key();
        }
		else{
			$outbound_redirect_uri = '';
		}

        if(!empty($mapping_id)){
            $this->db->update($this->table_name,$data,array('id' => $mapping_id),$format,array('%d'));
			if(!empty($outbound_redirect_uri)){
				$this->mo_sf_sync_save_data_in_meta_table($mapping_id,'outbound_redirect_uri',$outbound_redirect_uri);
			}
        } else {
            $this->db->insert($this->table_name,$data,$format);
            $mapping_id = $this->db->insert_id;
            if(!empty($outbound_redirect_uri))
                $this->mo_sf_sync_save_data_in_meta_table($mapping_id,'outbound_redirect_uri',$outbound_redirect_uri);
        }
        $this->mo_sf_sync_save_data_in_meta_table($mapping_id,'field_mapping',$field_map);
        return;
    }

    private function mo_sf_sync_create_key(){
        $token = bin2hex(random_bytes(8));
        return $token;
    }

    private function mo_sf_sync_if_mapping_already_exist(){
        $select_query = "SELECT id from $this->table_name LIMIT 1";
        $result = $this->db->get_results($select_query);

        if(!empty($result)){
            $id = $result[0]->id;
            return $id;
        }
        else{
            return false;
        }
    }

    public function mo_sf_sync_save_data_in_meta_table($mapping_id,$meta_key,$meta_value){
        $data = array(
            'mapping_id' => $mapping_id,
            'meta_key' => $meta_key,
            'meta_value' => maybe_serialize($meta_value)
        );
        $format = array('%d','%s','%s');
        $meta_exists = $this->mo_sf_sync_if_meta_exists($mapping_id,$meta_key);
        if($meta_exists!=false){
            $this->db->update($this->meta_table_name,$data,array('meta_key' => $meta_key),$format,array('%s'));
        } else {
            $this->db->insert($this->meta_table_name,$data,$format);
        }
    }

    private function mo_sf_sync_if_meta_exists($mapping_id,$meta_key){
        $select_query = "SELECT COUNT(*) from $this->meta_table_name where mapping_id='$mapping_id' AND meta_key='$meta_key'";
        $result = $this->db->get_results($select_query,ARRAY_A);
        if(isset($result[0]['COUNT(*)']) && $result[0]['COUNT(*)'] > 0)
            return true;
        else
            return false;
    }

    public function mo_sf_sync_get_all_mapping_data(){
        $select_query = "SELECT * from $this->table_name";
        $result = $this->db->get_results($select_query,ARRAY_A);
        if(isset($result[0]['id']))
            $id = $result[0]['id'];
        else
            return;
        $meta_data = $this->mo_sf_sync_get_data_from_metatable($id);
        $mapping_data = array_merge($result[0],maybe_unserialize($meta_data));
        return $mapping_data;
    }

    public function mo_sf_sync_get_data_from_metatable($mapping_id, $meta_key = ''){
        $select_query = "SELECT * from $this->meta_table_name where mapping_id = '$mapping_id'";
        if(!empty($meta_key))
            $select_query .= " AND meta_key = '$meta_key'";

        $result = $this->db->get_results($select_query,ARRAY_A);
        $ret = array();
        foreach($result as $key => $value)
            $ret[$value['meta_key']] = $value['meta_value'];
        
        return $ret;
    }

    private function mo_sf_sync_get_mapping_id_from_mapping_label($mapping_label){
        $select_query = "SELECT id from $this->table_name where label = '$mapping_label'";
        $result = $this->db->get_results($select_query,ARRAY_A);
        return $result[0]['id'];
    }

    public function mo_sf_sync_delete_data_from_metatable($mapping_id, $meta_key = ''){
        $delete_query = "DELETE FROM $this->meta_table_name where mapping_id = '$mapping_id'";
        if(!empty($meta_key))
            $delete_query += " AND meta_key = '$meta_key'";
        
        $this->db->get_results($delete_query);
        return;
    }

    public function mo_sf_sync_delete_mapping_from_db($mapping_label){
        $id = $this->mo_sf_sync_get_mapping_id_from_mapping_label($mapping_label);
        $this->mo_sf_sync_delete_data_from_metatable($id);
        $delete_query = "DELETE FROM $this->table_name WHERE label = '$mapping_label'";
        $this->db->get_results($delete_query);
    }

    public function mo_sf_sync_insert_imported_mapping_data($mapping_data){
        $data = array(
            'label' => $mapping_data['label'],
            'salesforce_object' => $mapping_data['salesforce_object'],
            'wordpress_object' => $mapping_data['wordpress_object'],
            'sync_sf_to_wp' => $mapping_data['sync_sf_to_wp'],
            'sync_wp_to_sf' => $mapping_data['sync_wp_to_sf']
        );
        $format = array('%s','%s','%s','%d','%d');
        $meta_key = $mapping_data['meta_key'];
        $meta_value = maybe_unserialize($mapping_data['field_mapping']);
        $get_query = "SELECT * from $this->table_name";
        $result = $this->db->get_results($get_query,ARRAY_A);
        if(is_array($result[0]) && !empty($result[0])){
            $this->db->update($this->table_name,$data,array('salesforce_object' => $result[0]['salesforce_object'],'wordpress_object' => $result[0]['wordpress_object']),$format,array('%s','%s'));
            $id = $result[0]['id'];
            $this->mo_sf_sync_save_data_in_meta_table($id,$meta_key,$meta_value);
        }
        else{
            $this->db->insert($this->table_name,$data,$format);
            $this->mo_sf_sync_save_data_in_meta_table($this->db->insert_id,$meta_key,$meta_value);
        }
    }
    public function mo_sf_sync_get_salesforce_record_id_for_wp_users(){
        $select_query = "SELECT salesforce_object FROM $this->table_name where wordpress_object = 'user' ";
		$data = $this->db->get_results($select_query);
        if(!empty($data))
            return 'salesforce_'.$data[0]->salesforce_object.'_ID' ;
        return array() ;
    }

    public function mo_sf_sync_get_salesforce_record_id_for_wp_posts(){
        $select_query = "SELECT salesforce_object FROM $this->table_name where wordpress_object = 'post' ";
		$data = $this->db->get_results($select_query);
        if(!empty($data))
            return 'salesforce_'.$data[0]->salesforce_object.'_ID' ;
        return array() ;
    }
    
}