<?php
namespace MoSfSyncSalesforce\Services;

use MoSfSyncSalesforce\Helper\PluginConstants;

class Utils{
	public static function mo_sf_sync_check_array_for_empty_or_null($arr_or_val){
		if(!is_array($arr_or_val)){
			if( !isset( $arr_or_val ) || empty( $arr_or_val )){
				return [
					'status' => 'error',
					'key' => $arr_or_val
				];
			}
		}else{
			foreach ($arr_or_val as $key => $value){
				if( !isset( $value ) || empty( $value )){
					return [
						'status' => 'error',
						'key' => $key
					];
				}
			}
		}
		return [
			'status' => 'success'
		];
	}

	public static function mo_sf_sync_save_settings($key,$value){
		return update_option($key, $value, false);
	}

	public static function mo_sf_sync_get_settings($key){
		return maybe_unserialize(get_option($key));
	}

	public static function mo_sf_sync_delete_settings($key){
		return delete_option($key);
	}

	public static function mo_sf_sync_sanitize_and_index($postData){
		$postData_sanitized = [];
		if(strpos($postData,PluginConstants::SELECTED_OBJECT) !== false){
			$postData = urldecode($postData);
			$postData_sanitized=explode('&',$postData);
			$i=0;
			while($i < count($postData_sanitized)){
				$postData_sanitized[$i]= explode('=',$postData_sanitized[$i]);
				$postData_sanitized[$i]= array($postData_sanitized[$i][0] , $postData_sanitized[$i][1]);
				$i++;
			}
			$non_field_mapping = 'object_select';
			$data_sanitized = [];
			foreach($postData_sanitized as $key => $values){
				if(empty($values[0]) || empty($values[1]) )
					continue;
				if($values[0] == $non_field_mapping)
					$data_sanitized[$values[0]] = $values[1];
				else if(substr($values[0], 0, 7) === "custom_")
					$data_sanitized['custom_field_mapping'][substr($values[0], 7)] = $values[1];
				else
					$data_sanitized['field_mapping'][$values[1]] = $values[0];
			}
			$data_sanitized['mapping_label'] = $data_sanitized['object_select'] . ' Map'; 
			return $data_sanitized;

		}else{
			parse_str($postData,$postData_sanitized);
			foreach ($postData_sanitized as $key => $value){
				if(empty($value)){
					unset($key);
				}else{
					$postData_sanitized[$key] = sanitize_text_field($value);
				}
			}
		}
		return $postData_sanitized;
	}

	public static function mo_sf_sync_manage_backward_compatability($mapping){

		foreach($mapping as $key => $value){
			if(empty($key) || empty($value))
				unset($mapping[$key]);
		}

		$new_mapping[PluginConstants::SELECTED_OBJECT] = $mapping[PluginConstants::SELECTED_OBJECT];
		unset($mapping[PluginConstants::SELECTED_OBJECT]);
		$new_mapping = array_merge($new_mapping, array_flip($mapping));
		self::mo_sf_sync_save_settings(PluginConstants::MAPPING_OBJECT, $new_mapping);
		return $new_mapping;
	}

	public static function mo_sf_sync_drop_table($table_name){
		global $wpdb;
		$select_query = $wpdb->prepare("DROP TABLE IF EXISTS `%s`",$table_name);
		$wpdb->get_results($select_query);
	}

	public static function mo_sf_sync_update_app_config($app_config){
		$app_config[PluginConstants::ENVIRONMENT] = "test";
		$app_config[PluginConstants::ENVIRONMENT_LINK] = "https://test.salesforce.com";
		Utils::mo_sf_sync_save_settings( PluginConstants::CONFIG_OBJECT, $app_config);
		return $app_config;
	}

	public static function mo_sf_sync_check_isset_empty($key, $array){
		if(!isset($array[$key]) || empty($array[$key]))
			return false;
		return true;
	}

	public static function mo_sf_sync_is_authorization_configured(){
		
		$sf_response = get_option('mo_sf_sync_salesforce_response');

		
		if(empty($sf_response) || !is_array($sf_response) )
			return false;
		
	
		if(!self::mo_sf_sync_check_isset_empty('access_token', $sf_response) || !self::mo_sf_sync_check_isset_empty('refresh_token', $sf_response))
			return false;

		return true;
	}

	public static function mo_sf_sync_show_feedback_message(){
		$message = self::mo_sf_sync_get_settings('mo_sf_sync_notice_message');
		if(empty($message))
			return;
		$message_type = self::mo_sf_sync_get_settings('mo_sf_sync_notice_message_type');
		echo "
			<div class='mo-sf-sync-alert mo-sf-sync-alert-$message_type mo-sf-mt-4' style='width:250px'>
			<div class='mo-sf-sync-popup-text' style='margin-top:15px'>
				<span class='dashicons dashicons-$message_type'></span>
				<span>$message</span>
			</div>
			</div>
		";
		self::mo_sf_sync_delete_settings('mo_sf_sync_notice_message');
		self::mo_sf_sync_delete_settings('mo_sf_sync_notice_message_type');
	}

	public static function mo_sf_sync_show_message(){
		$message = self::mo_sf_sync_get_settings('mo_sf_sync_information_message');
		if(empty($message))
			return;
		$message_type = self::mo_sf_sync_get_settings('mo_sf_sync_information_message_type');
		echo "
			<div class='mo-sf-sync-info-alert mo-sf-sync-info-info-alert-$message_type mo-sf-mt-4'>
				<span class='dashicons dashicons-$message_type'></span>
				<span>$message</span>
			</div>
		";
		self::mo_sf_sync_delete_settings('mo_sf_sync_information_message');
		self::mo_sf_sync_delete_settings('mo_sf_sync_information_message_type');
	}

	public static function mo_sf_sync_show_error_message($message){
        self::mo_sf_sync_save_settings('mo_sf_sync_notice_message',$message);
		self::mo_sf_sync_save_settings('mo_sf_sync_notice_message_type', 'warning');
    }

	
	public static function mo_sf_sync_information_message($message,$message_type){
        self::mo_sf_sync_save_settings('mo_sf_sync_information_message',$message);
		self::mo_sf_sync_save_settings('mo_sf_sync_information_message_type', 'warning');
    }


    public static function mo_sf_sync_show_success_message($message){
        self::mo_sf_sync_save_settings('mo_sf_sync_notice_message',$message);
		self::mo_sf_sync_save_settings('mo_sf_sync_notice_message_type', 'info');
    }

	public static function mo_sf_sync_display_config_related_message($message){
        self::mo_sf_sync_save_settings('mo_sf_sync_notice_message',$message);
		self::mo_sf_sync_save_settings('mo_sf_sync_notice_message_type', 'config-message');
    }

	public static function mo_sf_sync_show_error_for_updatable_fields($message){
		self::mo_sf_sync_save_settings('mo_sf_sync_updatable_fields_notice',$message);
	}

	public static function mo_sf_sync_show_updatable_fields_info(){
		$message = self::mo_sf_sync_get_settings('mo_sf_sync_updatable_fields_notice');
		if(empty($message))
			return;
		echo	'<section class="accordion mo-sf-mt-4 " >
					<input type="checkbox" name="collapse" checked="checked" >
					<h2 class="handle">
						<label class = "mo-sf-sync-coll-div-head mo_sf_sync_updatable_error" ><b>'.$message.'</b></label>
					</h2>	
					<div class="mo-sf-dflex mo-sf-sync-tab-content-tile content" style="border: none;box-shadow:none">
						<span><b>Note:</b> Non-Updatable Fields in Salesforce are the fields whose values cannot be updated through APIs.
							However you can change these fields from non-updatable to updatable from your Salesforce Account. After these fields
							are marked as updatable from salesforce, the plugin will allow you to map these fields to the desired WordPress Fields.
						<span><br/><br/>
						<span>Here are some links which can help you out in marking fields as updatable in Salesforce:</p>
							<ul style="list-style: disc;padding-left: 45px;">
								<li>Change the Field Type From Non Updatable to Updatable Using This <a href="">[LINK]</a></li>
								<li>Make Salesforce Object Accessible through REST APIs using This <a href="">[LINK]</a></li>
								<li>Change Persmission Sets For Users Using This <a href="">[LINK]</a></li>
							</ul>
						<span>Please reach out to us if you face any queries or if you have any difficulties.
						You can either mail us at <b>salesforcesupport@xecurify.com</b> or Submit your query to us using the support form adjacent to this information box.
						</span>
					</div>	
				</section>';
		self::mo_sf_sync_delete_settings('mo_sf_sync_updatable_fields_notice');
	}

	public static function mo_sf_sync_reset_plugin()
	{
		if( self::mo_sf_sync_check_option_admin_referer('mo_sf_sync_reset' )){
			
			delete_option(PluginConstants::CONFIG_OBJECT);
			delete_option(PluginConstants::MAPPING_OBJECT);
			delete_option(PluginConstants::PROVISION_OBJECT);
			delete_option(PluginConstants::SF_RESPONSE_OBJECT);
			delete_option(PluginConstants::AUDIT_LOGS);
			delete_option(PluginConstants::CONNECTION_TYPE);
			delete_transient('transient_lead_object');
			delete_transient('mo_sf_sync_made_trial_request');
			delete_transient('transient_get_object');
			delete_transient('mo_sf_sync_normal_trial_notice_dismiss_time');
			delete_transient('mo_sf_sync_integration_trial_notice_dismiss_time');
			Utils::mo_sf_sync_drop_table('mo_sf_sync_audit_log_db');
			Utils::mo_sf_sync_drop_table('mo_sf_sync_object_mapping_meta');
			Utils::mo_sf_sync_drop_table('mo_sf_sync_object_mapping');
		}
	}

	public static function mo_sf_sync_delete_mapping($mapping_label){
		$db = DBUtils::instance();
		$db->mo_sf_sync_delete_mapping_from_db($mapping_label);
	}

	public static function mo_sf_sync_check_option_admin_referer( $option_name ) {
		return ( isset( $_POST['option'] ) and $_POST['option'] == $option_name and check_admin_referer( $option_name ) );
	}

	public static function mo_sf_sync_wp_remote_call($url, $args = array(), $is_get=false){
		if(!$is_get)
			$response = wp_remote_post($url, $args);
		else
			$response = wp_remote_get($url, $args);
		if(!is_wp_error($response)){			
			return $response['body'];
		} else {
            self::mo_sf_sync_show_error_message('Unable to connect to the Internet. Please try again.');
            return false;
        }
    }

	public static function mo_sf_sync_check_empty_or_null( $value ) {
        if ( ! isset( $value ) || empty( $value ) ) {
            return true;
        }

        return false;
    }

	public static function mo_sf_sync_set_transient($key,$value,$expiration = 0){
		set_transient($key,$value,$expiration);
	}

	public static function mo_sf_sync_get_transient($key){
		get_transient($key);
	}

	public static function mo_sf_sync_class_exists_check(){
		$integrations = array();
		foreach(PluginConstants::INTEGRATIONS_ADVERTISEMENT as $key => $value){
			if(class_exists($key)){
				if(empty($integrations))
					$integrations = array($value);
				else
					array_push($integrations,$value);
			}
		}
		return $integrations;
	}

}
