<?php
/**
 * Plugin Name: Object Data Sync For Salesforce
 * Plugin URI: http://miniorange.com
 * Description: Object Data Sync For Salesforce Plugin synchronizes WordPress users with selected object/record in Salesforce and keeps Salesforce object/record in sync with the WordPress.
 * Version: 1.1.5
 * Author: miniOrange
 * Author URI: http://miniorange.com/
 * License: GPL2
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html 
 */

namespace MoSfSyncSalesforce;

use MoSfSyncSalesforce\Services\Utils;
use MoSfSyncSalesforce\Helper\Instance;
use MoSfSyncSalesforce\Handler\AjaxHandler;
use MoSfSyncSalesforce\Handler\AccountSetupHandler;
use MoSfSyncSalesforce\Handler\AuthorizationHandler;
use MoSfSyncSalesforce\Handler\BulkActionHandler;
use MoSfSyncSalesforce\Handler\DataProcessingHandler;
use MoSfSyncSalesforce\Helper\PluginConstants;
use MoSfSyncSalesforce\Handler\DemoSetupHandler;
use MoSfSyncSalesforce\Services\AuditDB;
use MoSfSyncSalesforce\Handler\WorkflowIntegrationHandler;
use MoSfSyncSalesforce\Handler\TestConfigurationHandler;
use MoSfSyncSalesforce\Services\DBUtils;

define('MOSF_DIRC', __DIR__);
include_once __DIR__ . '/Helper/view/view.php';
require( "FeedbackForm.php" );
require_once __DIR__ . '/vendor/autoload.php';
include_once __DIR__ . '/Helper/view/Troubleshoot.php';
include_once __DIR__ . '/Helper/view/PluginGuide.php';
include_once (__DIR__).'/Helper/view/templates/edit_template.php';
include_once (__DIR__).'/Helper/view/templates/table_template.php';
require_once __DIR__ .'/Handler/BulkActionHandler.php';

class MoSfSync{
	
	private static $instance;
	use Instance;
	private $dataProcessingHandler;

	public static function instance() {
		if ( ! isset( self::$instance ) ) {
			$class          = __CLASS__;
			self::$instance = new $class;
			self::$instance->mo_sf_sync_loadhooks();
			self::$instance->dataProcessingHandler = DataProcessingHandler::instance();
			new BulkActionHandler;
		}
		return self::$instance;
	}

	public function mo_sf_sync_loadhooks() {
		add_action('init', array($this, 'mo_sf_sync_handle_auth_code'));
		add_action('admin_init', array($this,'mo_sf_sync_save'));
		add_action('admin_menu', array($this,'mo_sf_sync_menu'));
		add_filter("bulk_actions-users", array($this,'mo_sf_sync_show_bulk_action'));
		add_filter("bulk_actions-edit-post", array($this,'mo_sf_sync_show_bulk_action_post'));
		add_action('admin_footer', array($this, 'mo_sf_sync_feedback_request'));
		add_action( 'restrict_manage_users', array($this,'mo_sf_sync_add_custom_search_filter'));
		add_action( 'restrict_manage_posts', array($this,'mo_sf_sync_add_custom_search_filter'));
		add_filter( 'pre_get_users', array($this,'mo_sf_sync_filter_users_by_sync_status') );
		add_filter( 'pre_get_posts', array($this,'mo_sf_sync_filter_users_by_sync_status') );
		add_action('wp_ajax_mo_sf_sync_ajax_submit', array($this, 'mo_sf_sync_ajax_submit_handler'));
		register_activation_hook(__FILE__, array($this, 'mo_sf_sync_plugin_init'));
		add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array($this,'mo_sf_sync_action_links') );
		
		$provision_config = Utils::mo_sf_sync_get_settings(PluginConstants::PROVISION_OBJECT);
		if(isset($provision_config[PluginConstants::AUTO_USER_UPDATE]) && $provision_config[PluginConstants::AUTO_USER_UPDATE] === 'on') {
			$this->mo_sf_sync_load_real_time_sync_hooks();
		}

		$this->mo_sf_sync_ad_hoc_sync_box();
	}
	function mo_sf_sync_action_links( $links ) {
		$url = esc_url( add_query_arg(
			'page',
			'mo_sf_sync',
			get_admin_url() . 'admin.php?page=mo_sf_sync'
		) );

		$license_link = "<a href='$url'>" . esc_html__( 'Settings' ) . '</a>';

		array_push(
			$links,
			$license_link
		);
		return $links;
	}

	public function mo_sf_sync_ad_hoc_sync_box() {
		add_action( 'edit_user_profile', array( $this, 'mo_sf_sync_show_salesforce_user_fields' ), 10, 1 );
		add_action( 'show_user_profile', array( $this, 'mo_sf_sync_show_salesforce_user_fields' ), 10, 1 );
		add_action( 'personal_options_update', array( $this, 'mo_sf_sync_update_show_salesforce_user_fields' ), 10, 1 );
		add_action( 'edit_user_profile_update', array( $this, 'mo_sf_sync_update_show_salesforce_user_fields' ), 10, 1 );
	}

	function mo_sf_sync_show_bulk_action($bulk_actions)
	{
		$db = DBUtils::instance();
		$nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_users();
		if (isset($nomenclature) && !empty($nomenclature)) 
			$bulk_actions["sync_user_to_sf"] = __('Sync Users to Salesforce', 'sync_user_to_sf');
		return $bulk_actions;
	}

	function mo_sf_sync_show_bulk_action_post($bulk_actions)
	{
		$db = DBUtils::instance();
		$nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_posts(); 
		if (isset($nomenclature) && !empty($nomenclature)) 
		    $bulk_actions["sync_post_to_sf"] = __('Sync Posts to Salesforce', 'sync_post_to_sf');	
		return $bulk_actions;
	}

	public function mo_sf_sync_show_salesforce_user_fields( $user ) {

		if ( ! current_user_can( 'manage_options' ) ) {
            return false;
        }

		$map = $this->dataProcessingHandler->mo_sf_sync_get_mapping($user->ID);
		$salesforce_object_id = "";
		$get_data= filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );

		if( isset( $get_data['edit_salesforce_mapping'] ) && true === filter_var( $get_data['edit_salesforce_mapping'], FILTER_VALIDATE_BOOLEAN ) ) {
			if (isset($map['object']))
				$salesforce_object_id = get_user_meta( $user->ID,"salesforce_".$map["object"]."_ID", true );
			
			$salesforce_id_temp = get_user_meta( $user->ID,"salesforce_ID", true );
			mo_sf_sync_edit_template($salesforce_id_temp,$salesforce_object_id);
		} else {	
			if(isset($get_data['push'] ) && true === filter_var( $get_data['push'], FILTER_VALIDATE_BOOLEAN )) {
				if (isset($map['object']))
					$salesforce_object_id = get_user_meta( $user->ID,"salesforce_".$map["object"]."_ID", true );
				
				$salesforce_id_temp=get_user_meta( $user->ID,"salesforce_ID", true );
				if(!empty($salesforce_id_temp) && isset($map['object']))
						update_user_meta( $user->ID, "salesforce_".$map["object"]."_ID", $salesforce_id_temp );
				
				$response = $this->dataProcessingHandler->mo_sf_sync_push_to_salesforce($user->ID);				
				$response = json_decode(json_encode($response),true);

				if(isset($response['message']) && !empty($response['message'])) {
					echo "<p style='color:darkred;font-size:15px;'><strong>Push Unsuccessful: ".esc_html($response['message'])."!</strong><br><p>";
				} else if(isset($response[0]['errorCode'])) {
					echo "<p style='color:darkred;font-size:15px;'><strong>Push Unsuccessful Salesforce returned error : ".esc_html($response[0]['message']) ."</strong><br><p>";
					if (array_key_exists("object",$map))
						update_user_meta( $user->ID, "salesforce_".$map["object"]."_ID", $salesforce_object_id );
				} else {
					echo "<p style='color:darkgreen;font-size:15px;'><strong>Push Successful</strong><br></p>";
				}
			}
           
			if (isset($map["object"]))
				$salesforce_object_id = get_user_meta( $user->ID,"salesforce_".$map["object"]."_ID", true );
			$salesforce_id_temp=get_user_meta( $user->ID,"salesforce_ID", true );
		
			mo_sf_sync_table_template($salesforce_id_temp,$salesforce_object_id,$user);
		}
	}
			
    public function mo_sf_sync_update_show_salesforce_user_fields( $user_id ) {
        if ( ! current_user_can( 'manage_options' ) ) {
            return false;
        }

        if ( array_key_exists( 'salesforce_ID' ,$_POST)) {
            update_user_meta( $user_id, 'salesforce_ID',  sanitize_text_field($_POST['salesforce_ID']) );
        }
    }

	public function mo_sf_sync_load_real_time_sync_hooks(){
		add_action('user_register', array($this, 'mo_sf_sync_update_salesforce'), 10, 1);
		add_action('profile_update', array($this, 'mo_sf_sync_update_salesforce'), 10, 1);
		add_action('save_post', array($this, 'mo_sf_sync_post_updated'), 10, 3);
	}

	public function mo_sf_sync_plugin_init(){
		$db = DBUtils::instance();
		$db->mo_sf_sync_create_mapping_tables();
		$mapping = Utils::mo_sf_sync_get_settings(PluginConstants::MAPPING_OBJECT);
		set_transient('mo_sf_sync_plugin_activated',true,86400);
		if(isset($mapping[PluginConstants::SELECTED_OBJECT]) && isset($mapping['user_login'])){
			Utils::mo_sf_sync_manage_backward_compatability($mapping);
		}
		$app_config = maybe_unserialize(Utils::mo_sf_sync_get_settings(PluginConstants::CONFIG_OBJECT));
		if(!isset($app_config[PluginConstants::ENVIRONMENT])){
			Utils::mo_sf_sync_update_app_config($app_config);
		}

		$this->mo_sf_sync_audit_log();
		$this->mo_sf_sync_activation_email();
	}

	public function mo_sf_sync_activation_email(){
		$email = get_option('admin_email');
		$message = 'Plugin Activated';

		$support = new Customer();
		$response = $support->mo_sf_sync_send_email_alert($email,"",$message);
	}

	public function mo_sf_sync_audit_log(){
		$audit = AuditDB::instance();
		$audit->mo_sf_sync_create_audit_log_table();
	}

	public function mo_sf_sync_add_custom_search_filter($which) {
		$db = DBUtils::instance();
		global $pagenow;
		if($pagenow === 'users.php')
		    $nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_users();
		else 
		    $nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_posts();
		if(!isset($nomenclature) || empty($nomenclature))
		    return;
		$st = '<select name="select_sync_status%s" style="float:none;margin-left:10px;">
			<option value="">%s</option>%s</select>';

		$options = '
			<option value="get_recs">Get Records</option>
			<option value="synced">Synced</option>
			<option value="not-synced">Not-Synced</option>
			';
		
		$select = sprintf( $st, $which, __( 'Sync Status ...' ), $options );

		echo $select;
		if($pagenow === 'users.php')
                   submit_button(__( 'Filter' ), null, $which, false);
	}
    public function mo_sf_sync_filter_users_by_sync_status($query) 
	{
		$db = DBUtils::instance();
		global $pagenow;
		
		if ( 'users.php' === $pagenow && is_admin()) 
			{
				$top = isset($_GET['select_sync_statustop']) ? sanitize_text_field($_GET['select_sync_statustop']) : null;
				$nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_users();
                if ( isset( $top ) && $top == 'not-synced' ) {
					$meta_query = array(
						array(
							'key' =>$nomenclature,
							'compare' => 'NOT EXISTS'
						),
					);
				}
				else if(isset( $top ) && $top == 'synced'){
					$meta_query = array(
						array(
							'key' =>$nomenclature,
							'compare' => 'EXISTS'
						),
					);
				}
			if (isset($meta_query)) {
				$query->set( 'meta_query', $meta_query );
			}	
		}
		else
			{
				$top = isset($_GET['select_sync_statuspost']) ? sanitize_text_field($_GET['select_sync_statuspost']) : null;
			
				$nomenclature = $db->mo_sf_sync_get_salesforce_record_id_for_wp_posts();
			
				if ( isset( $top ) && $top == 'not-synced' ) {
					$meta_query = array(
						array(
							'key' =>$nomenclature,
							'compare' => 'NOT EXISTS'
						),
					);
				
				}
				else if(isset( $top ) && $top == 'synced'){
				
					$meta_query = array(
						array(
							'key' =>$nomenclature,
							'compare' => 'EXISTS'
						),
					);
				
				}
				if (isset($meta_query)) {
					$query->set( 'meta_query', $meta_query );
				}	
		}
		return $query;
		// $_SERVER['REQUEST_URI'] = esc_url_raw(remove_query_arg(array('select_sync_statustop', 'action'), $_SERVER['REQUEST_URI']));		
	}
	//ToDo Move this logic to another file.
	public function mo_sf_sync_save(){

		if(isset($_POST['tab'])){		
			$tab = sanitize_text_field($_POST['tab']);
			switch ($tab){
				case 'account_setup':
					$handler = AccountSetupHandler::instance();
					$handler->mo_sf_sync_save_settings();
					break;

				case 'demo_setup':
					$handler = new DemoSetupHandler;
					$handler->mo_sf_sync_request_demo();
					break;
			}
		}
		
		if(isset($_POST['option'])){
			$option = sanitize_text_field($_POST['option']);
			switch ($option){
				case 'mo_sf_sync_delete':
					if(check_admin_referer('mo_sf_sync_delete')){
						$db = DBUtils::instance();
						$db->mo_sf_sync_delete_mapping_from_db(sanitize_text_field($_POST['mapping_label']));
					}
					break;
				case 'mo_sf_sync_reset':
					if(check_admin_referer('mo_sf_sync_reset'))
						Utils::mo_sf_sync_reset_plugin();
					break;
				case 'mo_sf_sync_feedback':
					if(!check_admin_referer('mo_sf_sync_feedback'))
						break;
					$miniorange_feedback_submit = sanitize_text_field($_POST['miniorange_feedback_submit']);
					$email= sanitize_text_field($_POST['query_mail']);
					$message = 'Plugin Deactivated';
					if($miniorange_feedback_submit == "Send"){
						$rate = sanitize_text_field($_POST['rate']);
						$query = sanitize_text_field($_POST['query_feedback']);
						$get_reply = isset($_POST['get_reply'])?'yes':'no';				
						$message .= ', [Reply:'.$get_reply.'], Feedback: '.$query.', [Rating: '.$rate.']';
					} else {
						$message .= ', Feedback: Skipped';
					}
					$support = new Customer();
					$response = $support->mo_sf_sync_send_email_alert($email,"",$message);
					deactivate_plugins( __FILE__ );
					wp_redirect('plugins.php');
					break;
				case 'mo_sf_sync_client_object':
					if(!check_admin_referer('mo_sf_sync_client_object','nonce_'))
						break;
					$handler = DataProcessingHandler::instance();
					$processed_data = $handler->mo_sf_sync_process_raw_mapping_data($_POST);
					break;
				case 'mo_sf_sync_client_config_option':
					$handler = AjaxHandler::instance();
					$handler->mo_sf_sync_handle_config_object_save($_POST);
					break;
				case 'mo_sf_sync_app_provisioning_config_option':
					$handler = DataProcessingHandler::instance();
					$processed_data = $handler->mo_sf_sync_save_realtime_sync($_POST);
					break;
				case 'mo_sf_sync_advanced_reports':
					if(!check_admin_referer('mo_sf_sync_advanced_reports'))
						break;
					$audit = AuditDB::instance();
					$audit->mo_sf_sync_save_advance_search_settings();
					break;
				case 'mo_sf_sync_clear_advance_search':
					if(!check_admin_referer('mo_sf_sync_clear_advance_search'))
						break;
					$audit = AuditDB::instance();
					$audit->mo_sf_sync_save_advance_search_settings();
					break;
				case 'mo_sf_sync_trial_request_for_integrations':
					if(!check_admin_referer('mo_sf_sync_trial_request_for_integrations'))
						break;
					
					$check_already_made_trial_request = get_transient('mo_sf_sync_made_integration_trial_request');
					if(!empty($check_already_made_trial_request) && $check_already_made_trial_request == true)
						Utils::mo_sf_sync_show_error_message('You have already made the integration trial request');
					else{
						$customer = new Customer();
						$response = $customer->mo_sf_sync_submit_contact_us(wp_get_current_user()->user_email, "", 'Can you please provide us trial version of the premium plugin.', false, true ,$_POST['requested_integrations'],'integration_trial_request');
						if($response == 'Query submitted.'){
							Utils::mo_sf_sync_show_success_message('Trial Request Successful');
							set_transient('mo_sf_sync_made_integration_trial_request',true,0);
						}
						else
							Utils::mo_sf_sync_show_error_message($response);
					}
					break;
				case 'mo_sf_sync_truncate_audit_logs':
					if(!check_admin_referer('mo_sf_sync_truncate_audit_logs'))
						break;
					$audit=AuditDB::instance();
					$audit->mo_sf_sync_clear_all_logs();
					break;
				case 'mo_sf_sync_setup_trial':
					if(!check_admin_referer('mo_sf_sync_setup_trial'))
						break;
					$check_already_made_trial_request = get_transient('mo_sf_sync_made_trial_request');
					if(!empty($check_already_made_trial_request) && $check_already_made_trial_request == true)
						Utils::mo_sf_sync_show_error_message('You have already made the trial request');
					else{
						$customer = new Customer();
						$response = $customer->mo_sf_sync_submit_contact_us(wp_get_current_user()->user_email, "", 'Can you please provide us trial version of the premium plugin.', false, true ,'None','normal_trial_request');
						if($response == 'Query submitted.'){
							Utils::mo_sf_sync_show_success_message('Trial Request Successful');
							set_transient('mo_sf_sync_made_trial_request',true,0);
						}
						else
							Utils::mo_sf_sync_show_error_message($response);
					}
					break;
			}
		}
	}

	public function mo_sf_sync_feedback_request(){
		mo_sf_sync_display_feedback_form();
	}

	public function mo_sf_sync_update_salesforce( $user_id ) {
		$response = $this->dataProcessingHandler->mo_sf_sync_push_to_salesforce($user_id);
	}

	public function mo_sf_sync_post_updated($id,$post_after,$post_before){
		$post_status = get_post_status($id);
		if( $post_status === 'publish' ){
			$response = $this->dataProcessingHandler->mo_sf_sync_push_to_salesforce($id, 'post');
		}
	}

	public function mo_sf_sync_menu() {
		add_menu_page('MiniOrange Salesforce sync' . __( 'Configure WP Salesforce Object Sync','mo_sf_sync' ),
			'Object Data Sync For Salesforce','administrator','mo_sf_sync', [$this, 'mo_sf_sync_display'], plugin_dir_url( __FILE__ ) . 'images/miniorange.png' );
		add_action( 'admin_enqueue_scripts', [$this, 'mo_sf_sync_settings_scripts' ] );
		add_action( 'admin_enqueue_scripts', [$this, 'mo_sf_sync_settings_style' ] );
		add_submenu_page('mo_sf_sync','Troubleshoot','Troubleshoot','administrator','troubleshoot',[$this, 'mo_sf_sync_troubleshoot']);
		add_submenu_page('mo_sf_sync','Configuration Guide','Configuration Guide','administrator','plugin_guide',[$this, 'mo_sf_sync_plugin_guide']);
	}

	public static function mo_sf_sync_troubleshoot(){
		mo_sf_sync_display_trouble_shoot();
	}

	public static function mo_sf_sync_plugin_guide(){
		mo_sf_sync_display_plugin_guide();
	}

	public static function mo_sf_sync_display() {
		mo_sf_sync_display_view();
	}

	public function mo_sf_sync_settings_scripts($page){
		if( $page != 'toplevel_page_mo_sf_sync' && $page != 'object-data-sync-for-salesforce_page_troubleshoot' ){
			return;
		}
		wp_enqueue_script('jquery');
		wp_enqueue_script('mo_sf_sync_LicensePlan',plugins_url('/Helper/view/includes/js/mo_sf_sync_license_plan.min.js',__FILE__),array('jquery'), PluginConstants::VERSION);
		wp_enqueue_script('selectwoo.js',plugins_url('/Helper/view/includes/js/vendor/selectwoo.min.js',__FILE__),['jquery'], PluginConstants::VERSION);
		wp_enqueue_script('mo_sf_sync_save_settings_ajax',plugins_url('/Helper/view/includes/js/mo_sf_sync_settings.min.js',__FILE__),['jquery','selectwoo.js'], PluginConstants::VERSION);
		wp_localize_script( 'mo_sf_sync_save_settings_ajax', 'ajax_object_sf', array( 'ajax_url_sf' => admin_url( '/admin-ajax.php' ) ) );
		wp_enqueue_script( 'mo_sf_sync_datatable_script', plugins_url('Helper/view/includes/js/jquery.dataTables.min.js', __FILE__ ), array('jquery'));
	}

	public function mo_sf_sync_settings_style($page){
		if( $page != 'toplevel_page_mo_sf_sync' && $page != 'object-data-sync-for-salesforce_page_troubleshoot' && $page != 'object-data-sync-for-salesforce_page_plugin_guide'){
			return;
		}
		$css_url = plugins_url('/Helper/view/includes/css/mo_sf_sync_settings.min.css',__FILE__);
		wp_enqueue_style('selectwoo.css',plugins_url('/Helper/view/includes/css/vendor/selectwoo.min.css',__FILE__),array(), PluginConstants::VERSION);
		wp_enqueue_style('mo_sf_sync_css',$css_url,array('selectwoo.css'), PluginConstants::VERSION);
		wp_enqueue_style('license_style_css',plugins_url('/Helper/view/includes/css/license_style.min.css',__FILE__),array(), PluginConstants::VERSION);
		wp_enqueue_style( 'mo_sf_sync_datatable_style', plugins_url('Helper/view/includes/css/jquery.dataTables.min.css', __FILE__));
	}

	public function mo_sf_sync_ajax_submit_handler(){
		$handler = AjaxHandler::instance();
		$handler->mo_sf_sync_set_settings();
	}

	public function mo_sf_sync_handle_auth_code(){
		
		if(array_key_exists('option',$_GET)){
			$option = sanitize_text_field($_GET['option']);
			switch ($option){
				case 'authorization_flow':
					$auth_handler = AuthorizationHandler::instance();
					$auth_handler->mo_sf_sync_connect_to_salesforce();
					break;
				case 'mo_sf_sync_test_connection':
					$wp_id = $_GET['mo_sf_sync_wpid'];
					$handler = TestConfigurationHandler::instance();
					$handler->mo_sf_sync_show_test_connection_window($wp_id);
					break;
			}
		}
		if(isset($_REQUEST['method']) && $_REQUEST['method'] == 'soap'){
			$handler = new WorkflowIntegrationHandler();
			$handler->mo_sf_sync_xml_parser($_REQUEST);
		} 
		$handler = AuthorizationHandler::instance();
		$handler->mo_sf_sync_handle_code();
	}
}

MoSfSync::instance();